@@ -1,9 +1,10 @@
 /*
- * Copyright 2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -29,7 +30,7 @@ import org.apache.tools.ant.Project;
  * by the mod_jk status (1.2.13) application.
  * 
  * @author Peter Rossbach
- * @version $Revision:$
+ * @version $Revision: 550245 $
  * @since 5.5.10
  */
 public class JkStatusTask extends BaseRedirectorHelperTask {
@@ -37,7 +38,7 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
     /**
      * The descriptive information about this implementation.
      */
-    private static final String info = "org.apache.jk.status.JkStatusTask/1.1";
+    private static final String info = "org.apache.jk.status.JkStatusTask/1.2";
 
     /**
      * Store status as <code>resultProperty</code> prefix.
@@ -180,17 +181,34 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
         try {
             JkStatusAccessor accessor = new JkStatusAccessor();
             JkStatus status = accessor.status(url, username, password);
+            if (status.result != null && !"OK".equals(status.result.type) ) {
+                if (getErrorProperty() != null) {
+                    getProject().setNewProperty(errorProperty, status.result.message);
+                }
+                if (isFailOnError()) {
+                    throw new BuildException(status.result.message);
+                } else {
+                    handleErrorOutput(status.result.message);
+                    return;
+                }
+
+            }
+            
             if (!isWorkerOnly && !isLoadbalancerOnly) {
                 JkServer server = status.getServer();
+                JkSoftware software = status.getSoftware();
+                JkResult result = status.getResult();
                 if (resultproperty != null) {
-                    createProperty(server, "name");
-                    createProperty(server, "port");
-                    createProperty(server, "version");
-                    createProperty(server, "software");
+                    createProperty(server, "server", "name");
+                    createProperty(server, "server", "port");
+                    createProperty(software, "web_server");
+                    createProperty(software, "jk_version");
+                    createProperty(result, "result", "type");
+                    createProperty(result, "result", "message");
                 }
                 if (isEcho()) {
-                    handleOutput("server name=" + server.getName() + "."
-                            + server.getPort() + " - " + server.getSoftware());
+                    handleOutput("server name=" + server.getName() + ":"
+                            + server.getPort() + " - " + software.getWeb_server() + " - " + software.getJk_version());
                 }
             }
             List balancers = status.getBalancers();
@@ -207,9 +225,13 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
                     }
                 } else {
                     if (!isWorkerOnly) {
-                        if (resultproperty != null) {
-                            balancerIndex = Integer.toString(balancer.getId());
-                            setPropertyBalancer(balancer,balancerIndex);
+                        if (resultproperty != null) { 
+                        	if ( balancer.getId() >= 0)
+                        		balancerIndex = Integer.toString(balancer.getId());
+                        	else
+                        		balancerIndex = balancer.getName() ;
+                        	
+                        	setPropertyBalancer(balancer,balancerIndex);
                         }
                         echoBalancer(balancer);
                     }
@@ -228,7 +250,7 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
                             }
                         } else {
                             if (resultproperty != null) {
-                                setPropertyWorker(balancerIndex, member);
+                                setPropertyWorker(null, member);
                             }
                             echoWorker(member);
                             if (member.getStatus() != null && !"OK".equals(member.getStatus())) {
@@ -236,7 +258,10 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
                                         + " status=" + member.getStatus()
                                         + " host=" + member.getAddress());
                             }
-                            if (member.getState() != null && !("OK".equals(member.getState()) || "N/A".equals(member.getState())) ){
+                            if (member.getState() != null &&
+                                !("OK".equals(member.getState()) ||
+                                  "N/A".equals(member.getState()) ||
+                                  "OK/IDLE".equals(member.getState())) ){
                                 error.append(" worker name=" + member.getName()
                                         + " state=" + member.getState()
                                         + " host=" + member.getAddress());
@@ -246,37 +271,61 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
                     if (!isWorkerOnly) {
                         if (resultproperty != null && members.size() > 0) {
                             getProject().setNewProperty(
-                                    resultproperty + ".balancer."
-                                            + balancerIndex + ".member.length",
+                                    resultproperty + "."
+                                            + balancer.getName() + ".length",
                                     Integer.toString(members.size()));
                         }
                         List mappings = balancer.getBalancerMappings();
                         int j = 0;
+                        String mapIndex ;
+                        if( balancerIndex != null )
+                        	mapIndex = balancerIndex + ".map" ;
+                        else
+                        	mapIndex = "map" ;
                         for (Iterator iterator = mappings.iterator(); iterator
                                 .hasNext(); j++) {
                             JkBalancerMapping mapping = (JkBalancerMapping) iterator
                                     .next();
                             if (resultproperty != null) {
-                                String stringIndex2 = Integer.toString(j);
-                                createProperty(mapping, balancerIndex,
+                            	String stringIndex2 ;
+                                if( mapping.getId() >= 0) {
+                                    stringIndex2 =  Integer.toString(mapping.getId()) ;
+                                } else {
+                                    stringIndex2 = Integer.toString(j);
+                                }                       	
+                                createProperty(mapping, mapIndex,
                                         stringIndex2, "type");
-                                createProperty(mapping, balancerIndex,
+                                createProperty(mapping, mapIndex,
                                         stringIndex2, "uri");
-                                createProperty(mapping, balancerIndex,
+                                createProperty(mapping, mapIndex,
                                         stringIndex2, "context");
+                                createProperty(mapping, mapIndex,
+                                        stringIndex2, "source");
                             }
                             if (isEcho()) {
-                                handleOutput("balancer name="
+                            	String mappingOut ;
+                            	
+                            	if(mapping.source != null) {
+                            		mappingOut =
+                            			"balancer name="
+                                        + balancer.getName() + " mappingtype="
+                                        + mapping.getType() + " uri="
+                                        + mapping.getUri() + " source="
+                                        + mapping.getSource() ;
+                            	} else {
+                            		mappingOut = "balancer name="
                                         + balancer.getName() + " mappingtype="
                                         + mapping.getType() + " uri="
                                         + mapping.getUri() + " context="
-                                        + mapping.getContext());
+                                        + mapping.getContext() ;
+                            	}
+                            	handleOutput(mappingOut);
                             }
                         }
                         if (resultproperty != null && mappings.size() > 0) {
                             getProject().setNewProperty(
-                                    resultproperty + ".balancer."
-                                            + balancerIndex + ".map.length",
+                                    resultproperty + "."
+                                            + mapIndex + ".length",
                                     Integer.toString(mappings.size()));
                         }
                     }
@@ -285,7 +334,7 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
             if (!isWorkerOnly && !isLoadbalancerOnly) {
                 if (resultproperty != null && balancers.size() > 0) {
                     getProject().setNewProperty(
-                            resultproperty + ".balancer.length",
+                            resultproperty + ".length",
                             Integer.toString(balancers.size()));
                 }
             }
@@ -350,17 +399,42 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
      */
     private void setPropertyBalancerOnly(JkBalancer balancer) {
         String prefix = resultproperty + "." + balancer.getName();
-        getProject().setNewProperty(prefix + ".id",
+        if(balancer.getId() >= 0 ) {
+        	getProject().setNewProperty(prefix + ".id",
                 Integer.toString(balancer.getId()));
+        }
         getProject().setNewProperty(prefix + ".type", balancer.getType());
-        getProject().setNewProperty(prefix + ".sticky",
-                Boolean.toString(balancer.isSticky()));
-        getProject().setNewProperty(prefix + ".stickyforce",
-                Boolean.toString(balancer.isStickyforce()));
+        getProject().setNewProperty(prefix + ".stick_session",
+                Boolean.toString(balancer.isSticky_session()));
+        getProject().setNewProperty(prefix + ".sticky_session_force",
+                Boolean.toString(balancer.isSticky_session_force()));
         getProject().setNewProperty(prefix + ".retries",
                 Integer.toString(balancer.getRetries()));
-        getProject().setNewProperty(prefix + ".recover",
-                Integer.toString(balancer.getRecover()));
+        getProject().setNewProperty(prefix + ".recover_time",
+                Integer.toString(balancer.getRecover_time()));
+        getProject().setNewProperty(prefix + ".method",
+                balancer.getMethod());
+        getProject().setNewProperty(prefix + ".good",
+                Integer.toString(balancer.getGood()));
+        getProject().setNewProperty(prefix + ".degraded",
+                Integer.toString(balancer.getDegraded()));
+        getProject().setNewProperty(prefix + ".bad",
+                Integer.toString(balancer.getBad()));
+        getProject().setNewProperty(prefix + ".busy",
+                Integer.toString(balancer.getBusy()));
+        getProject().setNewProperty(prefix + ".map_count",
+                Integer.toString(balancer.getMap_count()));
+        getProject().setNewProperty(prefix + ".member_count",
+                Integer.toString(balancer.getMember_count()));
+        getProject().setNewProperty(prefix + ".max_busy",
+                Integer.toString(balancer.getMax_busy()));
+        getProject().setNewProperty(prefix + ".time_to_maintenance_min",
+                Integer.toString(balancer.getTime_to_maintenance_min()));
+        getProject().setNewProperty(prefix + ".time_to_maintenance_max",
+                Integer.toString(balancer.getTime_to_maintenance_max()));
+        getProject().setNewProperty(prefix + ".lock",
+                balancer.getLock());
+ 
     }
 
     /**
@@ -368,23 +442,64 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
      * @return
      */
     private void setPropertyBalancer(JkBalancer balancer,String balancerIndex) {
-        createProperty(balancer, balancerIndex, "id");
+     	if(balancer.id >= 0) {
+    		createProperty(balancer, balancerIndex, "id");
+    	}
+
         createProperty(balancer, balancerIndex, "name");
         createProperty(balancer, balancerIndex, "type");
-        createProperty(balancer, balancerIndex, "sticky");
-        createProperty(balancer, balancerIndex, "stickyforce");
+        createProperty(balancer, balancerIndex, "sticky_session");
+        createProperty(balancer, balancerIndex, "sticky_session_force");
         createProperty(balancer, balancerIndex, "retries");
-        createProperty(balancer, balancerIndex, "recover");
-    }
+        createProperty(balancer, balancerIndex, "recover_time");
+        if(balancer.getMethod() != null) {
+        	createProperty(balancer, balancerIndex, "method");
+        }
+        if(balancer.getLock() != null) {
+        	createProperty(balancer, balancerIndex, "lock");
+        }
+        if(balancer.getGood() >= 0) {
+        	createProperty(balancer, balancerIndex, "good");
+        }
+        if(balancer.getDegraded() >= 0) {
+        	createProperty(balancer, balancerIndex, "degraded");
+        }
+        if(balancer.getBad() >= 0) {
+        	createProperty(balancer, balancerIndex, "bad");
+        }
+        if(balancer.getBusy() >= 0) {
+        	createProperty(balancer, balancerIndex, "busy");
+        }
+        if(balancer.getMax_busy() >= 0) {
+        	createProperty(balancer, balancerIndex, "max_busy");
+        }
+        if(balancer.getMember_count() >=0) {
+        	createProperty(balancer, balancerIndex, "member_count");
+        }
+        if(balancer.getMap_count() >=0) {
+        	createProperty(balancer, balancerIndex, "map_count");
+        }
+        if(balancer.getTime_to_maintenance_min() >=0) {
+        	createProperty(balancer, balancerIndex, "time_to_maintenance_min");
+        }
+        if(balancer.getTime_to_maintenance_max() >=0) {
+        	createProperty(balancer, balancerIndex, "time_to_maintenance_max");
+        }
+   }
 
     /**
      * @param balancerIndex
      * @param member
      */
     private void setPropertyWorker(String balancerIndex, JkBalancerMember member) {
-        String workerIndex = Integer.toString(member.getId());
-        createProperty(member, balancerIndex, workerIndex, "id");
-        createProperty(member, balancerIndex, workerIndex, "name");
+        String workerIndex ;
+        if(member.getId() >= 0) {
+        	workerIndex = Integer.toString(member.getId());
+            createProperty(member, balancerIndex, workerIndex, "id");
+            createProperty(member, balancerIndex, workerIndex, "name");
+        } else {
+        	workerIndex = member.getName();
+        }
         createProperty(member, balancerIndex, workerIndex, "type");
         createProperty(member, balancerIndex, workerIndex, "host");
         createProperty(member, balancerIndex, workerIndex, "port");
@@ -392,6 +507,9 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
         if(member.getJvm_route() != null) {
             createProperty(member, balancerIndex, workerIndex, "jvm_route");
         }
+        if(member.getRoute() != null) {
+            createProperty(member, balancerIndex, workerIndex, "route");
+        }       
         if(member.getStatus() != null) {
             createProperty(member, balancerIndex, workerIndex, "status");
         }
@@ -403,33 +521,33 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
         }
         createProperty(member, balancerIndex, workerIndex, "lbfactor");
         createProperty(member, balancerIndex, workerIndex, "lbvalue");
-        if(member.getLbmult() > 0) {
+        if(member.getLbmult() >= 0) {
             createProperty(member, balancerIndex, workerIndex, "lbmult");
         }
         createProperty(member, balancerIndex, workerIndex, "elected");
         createProperty(member, balancerIndex, workerIndex, "readed");
         createProperty(member, balancerIndex, workerIndex, "busy");
-        if(member.getMaxbusy() > 0) {
-            createProperty(member, balancerIndex, workerIndex, "maxbusy");
+        if(member.getMax_busy() >= 0) {
+            createProperty(member, balancerIndex, workerIndex, "max_busy");
         }
         createProperty(member, balancerIndex, workerIndex, "transferred");
         createProperty(member, balancerIndex, workerIndex, "errors");
-        if(member.getClienterrors() > 0) {
-            createProperty(member, balancerIndex, workerIndex, "clienterrors");
+        if(member.getClient_errors() >= 0) {
+            createProperty(member, balancerIndex, workerIndex, "client_errors");
         }
-        if(member.getDistance() > 0) {
+        if(member.getDistance() >= 0) {
             createProperty(member, balancerIndex, workerIndex, "distance");
         }
         if (member.getDomain() != null) {
             createProperty(member, balancerIndex, workerIndex, "domain");
         } else {
-            getProject().setNewProperty(resultproperty + ".balancer." + balancerIndex + ".member." + workerIndex +
+            getProject().setNewProperty(resultproperty + "." + balancerIndex + "." + workerIndex +
                     ".domain", "");          
         }
         if (member.getRedirect() != null) {
             createProperty(member, balancerIndex, workerIndex, "redirect");
         } else {
-            getProject().setNewProperty(resultproperty + ".balancer." + balancerIndex + ".member." + workerIndex +
+            getProject().setNewProperty(resultproperty + "." + balancerIndex + "." + workerIndex +
                     ".redirect", "");          
         }
     }
@@ -440,25 +558,32 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
      */
     private void setPropertyWorkerOnly(JkBalancer balancer,
             JkBalancerMember member) {
+        //String prefix = resultproperty + "." + balancer.getName() + "." + member.getName();
         String prefix = resultproperty + "." + member.getName();
-        Project currentProject = getProject();
-        
-        currentProject.setNewProperty(prefix + ".lb.id",
+               Project currentProject = getProject();
+        if ( balancer.getId() >= 0) { 
+        	currentProject.setNewProperty(prefix + ".lb.id",
                 Integer.toString(balancer.getId()));
-        currentProject.setNewProperty(prefix + ".lb.name", balancer.getName());
-        currentProject.setNewProperty(prefix + ".id",
+        }
+        //currentProject.setNewProperty(prefix + ".lb.name", balancer.getName());
+        if( member.getId() >= 0) {
+        	currentProject.setNewProperty(prefix + ".id",
                 Integer.toString(member.getId()));
+        }
         currentProject.setNewProperty(prefix + ".type", member.getType());
-        if(member.getJvm_route() != null) {
+        if (member.getJvm_route() != null) {
             currentProject.setNewProperty(prefix + ".jvm_route", member.getJvm_route());
         }
-        if(member.getStatus() != null) {
+        if (member.getRoute() != null) {
+            currentProject.setNewProperty(prefix + ".route", member.getRoute());
+        }
+        if (member.getStatus() != null) {
             currentProject.setNewProperty(prefix + ".status", member.getStatus());
         }
-        if(member.getActivation() != null) {
+        if (member.getActivation() != null) {
             currentProject.setNewProperty(prefix + ".activation", member.getActivation());
         }
-        if(member.getState() != null) {
+        if (member.getState() != null) {
             currentProject.setNewProperty(prefix + ".state", member.getState());
         }
         currentProject.setNewProperty(prefix + ".host", member.getHost());
@@ -469,7 +594,7 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
                 Integer.toString(member.getLbfactor()));
         currentProject.setNewProperty(prefix + ".lbvalue",
                 Long.toString(member.getLbvalue()));
-        if(member.getLbmult() > 0) {
+        if(member.getLbmult() >= 0) {
             currentProject.setNewProperty(prefix + ".lbmult",
                     Long.toString(member.getLbmult()));
         }
@@ -481,29 +606,46 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
                 Long.toString(member.getTransferred()));
         currentProject.setNewProperty(prefix + ".busy",
                 Integer.toString(member.getBusy()));
-        if(member.getMaxbusy() > 0) {
-            currentProject.setNewProperty(prefix + ".maxbusy",
-                    Long.toString(member.getMaxbusy()));
+        if(member.getMax_busy() >= 0) {
+            currentProject.setNewProperty(prefix + ".max_busy",
+                    Long.toString(member.getMax_busy()));
         }
         currentProject.setNewProperty(prefix + ".errors",
                 Long.toString(member.getErrors()));
-        if(member.getClienterrors() > 0) {
-            currentProject.setNewProperty(prefix + ".clienterrors",
-                    Long.toString(member.getClienterrors()));
+        if(member.getClient_errors() >= 0) {
+            currentProject.setNewProperty(prefix + ".client_errors",
+                    Long.toString(member.getClient_errors()));
         }
-        if(member.getDistance() > 0) {
+        if(member.getDistance() >= 0) {
             currentProject.setNewProperty(prefix + ".distance",
                     Integer.toString(member.getDistance()));
         }
-        if (member.getDomain() != null)
+        if (member.getDomain() != null) {
             currentProject.setNewProperty(prefix + ".domain", member.getDomain());
-        else
+        } else {
             currentProject.setNewProperty(prefix + ".domain", "");
-        if (member.getRedirect() != null)
+        }
+        if (member.getRedirect() != null) {
             currentProject.setNewProperty(prefix + ".redirect",
                     member.getRedirect());
-        else
+        } else {
             currentProject.setNewProperty(prefix + ".redirect", "");
+        }
+        if(member.getTime_to_recover() >= 0) {
+            currentProject.setNewProperty(prefix + ".time_to_recover",
+                    Integer.toString(member.getTime_to_recover()));
+        }
+        if(member.getTime_to_recover_min() >= 0) {
+            currentProject.setNewProperty(prefix + ".time_to_recover_min",
+                    Integer.toString(member.getTime_to_recover_min()));
+        }
+        if(member.getTime_to_recover_max() >= 0) {
+            currentProject.setNewProperty(prefix + ".time_to_recover_max",
+                    Integer.toString(member.getTime_to_recover_max()));
+            currentProject.setNewProperty(prefix + ".time_to_recover",
+                    (Integer.toString((member.getTime_to_recover_min()
+                            + member.getTime_to_recover_max()) / 2)));
+        }
             
     }
 
@@ -540,36 +682,43 @@ public class JkStatusTask extends BaseRedirectorHelperTask {
             String arraymark2, String attribute) {
         if (resultproperty != null) {
             Object value = IntrospectionUtils.getProperty(result, attribute);
-            if (value != null) {
+            if (value != null ) {
                 StringBuffer propertyname = new StringBuffer(resultproperty);
 
-                if (result instanceof JkServer) {
-                    propertyname.append(".server");
-                } else if (result instanceof JkBalancer) {
-                    propertyname.append(".balancer");
+                if (result instanceof JkBalancer) {
                     if (arraymark != null) {
                         propertyname.append(".");
                         propertyname.append(arraymark);
                     }
+                } else if (result instanceof JkServer) {
+					if (arraymark != null) {
+						propertyname.append(".");
+						propertyname.append(arraymark);
+					}
+                } else if (result instanceof JkSoftware) {
+					if (arraymark != null) {
+						propertyname.append(".");
+						propertyname.append(arraymark);
+					}
+				} else if (result instanceof JkResult) {
+					if (arraymark != null) {
+						propertyname.append(".");
+						propertyname.append(arraymark);
+					}
                 } else if (result instanceof JkBalancerMember) {
-                    propertyname.append(".balancer");
                     if (arraymark != null) {
                         propertyname.append(".");
                         propertyname.append(arraymark);
                     }
-                    propertyname.append(".member");
                     if (arraymark2 != null) {
                         propertyname.append(".");
                         propertyname.append(arraymark2);
                     }
-
                 } else if (result instanceof JkBalancerMapping) {
-                    propertyname.append(".balancer");
                     if (arraymark != null) {
                         propertyname.append(".");
                         propertyname.append(arraymark);
                     }
-                    propertyname.append(".map");
                     if (arraymark2 != null) {
                         propertyname.append(".");
                         propertyname.append(arraymark2);
