@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -22,7 +23,7 @@
  * @package jk_connect
  * @author      Gal Shachor <shachor@il.ibm.com>
  * @author      Mladen Turk <mturk@apache.org>
- * @version     $Revision: 440792 $
+ * @version     $Revision: 555199 $
  */
 
 
@@ -119,7 +120,7 @@ static int sononblock(jk_sock_t sd)
 static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
 {
     int rc;
-    if (timeout < 1)
+    if (timeout <= 0)
         return connect(sock, addr, sizeof(struct sockaddr_in));
 
     if ((rc = sononblock(sock)))
@@ -221,6 +222,27 @@ static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
 }
 #endif
 
+
+#ifdef AS400_UTF8
+
+/*
+ *  i5/OS V5R4 need EBCDIC for its runtime calls but APR/APACHE works in UTF
+ */
+in_addr_t jk_inet_addr(const char * addrstr)
+{
+    in_addr_t addr;
+    char *ptr;
+
+    ptr = (char *)malloc(strlen(addrstr) + 1);
+    jk_ascii2ebcdic((char *)addrstr, ptr);
+    addr = inet_addr(ptr);
+    free(ptr);
+
+    return(addr);
+}
+
+#endif
+
 /** resolve the host IP */
 
 int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
@@ -235,7 +257,7 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
 
     /* Check if we only have digits in the string */
     for (x = 0; host[x] != '\0'; x++) {
-        if (!isdigit(host[x]) && host[x] != '.') {
+        if (!isdigit((int)(host[x])) && host[x] != '.') {
             break;
         }
     }
@@ -269,7 +291,8 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
             return JK_FALSE;
 
         apr_sockaddr_ip_get(&remote_ipaddr, remote_sa);
-        laddr.s_addr = inet_addr(remote_ipaddr);
+
+        laddr.s_addr = jk_inet_addr(remote_ipaddr);
 
 #else /* HAVE_APR */
 
@@ -290,7 +313,7 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
     }
     else {
         /* If we found only digits we use inet_addr() */
-        laddr.s_addr = inet_addr(host);
+        laddr.s_addr = jk_inet_addr(host);
     }
     memcpy(&(rc->sin_addr), &laddr, sizeof(laddr));
 
@@ -316,7 +339,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
     if (!IS_VALID_SOCKET(sock)) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
-               "socket() failed with errno=%d", errno);
+               "socket() failed (errno=%d)", errno);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
 ;
@@ -325,7 +348,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
     if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (SET_TYPE)&set,
                    sizeof(set))) {
         jk_log(l, JK_LOG_ERROR,
-                "failed setting TCP_NODELAY with errno=%d", errno);
+                "failed setting TCP_NODELAY (errno=%d)", errno);
         jk_close_socket(sock);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
@@ -338,7 +361,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
         if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (SET_TYPE)&set,
                        sizeof(set))) {
             jk_log(l, JK_LOG_ERROR,
-                   "failed setting SO_KEEPALIVE with errno=%d", errno);
+                   "failed setting SO_KEEPALIVE (errno=%d)", errno);
             jk_close_socket(sock);
             JK_TRACE_EXIT(l);
             return JK_INVALID_SOCKET;
@@ -355,7 +378,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                         sizeof(set))) {
             JK_GET_SOCKET_ERRNO();
             jk_log(l, JK_LOG_ERROR,
-                    "failed setting SO_SNDBUF with errno=%d", errno);
+                    "failed setting SO_SNDBUF (errno=%d)", errno);
             jk_close_socket(sock);
             JK_TRACE_EXIT(l);
             return JK_INVALID_SOCKET;
@@ -366,7 +389,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                                 sizeof(set))) {
             JK_GET_SOCKET_ERRNO();
             jk_log(l, JK_LOG_ERROR,
-                    "failed setting SO_RCVBUF with errno=%d", errno);
+                    "failed setting SO_RCVBUF (errno=%d)", errno);
             jk_close_socket(sock);
             JK_TRACE_EXIT(l);
             return JK_INVALID_SOCKET;
@@ -408,7 +431,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                    sizeof(int))) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
-                "failed setting SO_NOSIGPIPE with errno=%d", errno);
+                "failed setting SO_NOSIGPIPE (errno=%d)", errno);
         jk_close_socket(sock);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
@@ -421,7 +444,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                    sizeof(li))) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
-                "failed setting SO_LINGER with errno=%d", errno);
+                "failed setting SO_LINGER (errno=%d)", errno);
         jk_close_socket(sock);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
@@ -448,7 +471,7 @@ iSeries when Unix98 is required at compil time */
     /* Check if we are connected */
     if (ret) {
         jk_log(l, JK_LOG_INFO,
-               "connect to %s failed with errno=%d",
+               "connect to %s failed (errno=%d)",
                jk_dump_hinfo(addr, buf), errno);
         jk_close_socket(sock);
         sock = JK_INVALID_SOCKET;
@@ -466,21 +489,19 @@ iSeries when Unix98 is required at compil time */
 
 int jk_close_socket(jk_sock_t s)
 {
+    if (IS_VALID_SOCKET(s))
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    if (s != INVALID_SOCKET)
         return closesocket(s) ? -1 : 0;
 #else
-    if (s != -1)
         return close(s);
 #endif
-
     return -1;
 }
 
 #ifndef MAX_SECS_TO_LINGER
-#define MAX_SECS_TO_LINGER 16
+#define MAX_SECS_TO_LINGER 30
 #endif
-#define SECONDS_TO_LINGER  1
+#define SECONDS_TO_LINGER  2
 
 #ifndef SHUT_WR
 #ifdef SD_SEND
@@ -491,17 +512,13 @@ int jk_close_socket(jk_sock_t s)
 #endif
 int jk_shutdown_socket(jk_sock_t s)
 {
-    unsigned char dummy[512];
-    int nbytes;
-    int ttl = 0;
+    char dummy[512];
     int rc = 0;
-#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    int tmout = SECONDS_TO_LINGER * 1000;
-    if (s == INVALID_SOCKET)
-#else
+    fd_set rs;
     struct timeval tv;
-    if (s < 0)
-#endif
+    time_t start = time(NULL);
+
+    if (!IS_VALID_SOCKET(s))
         return -1;
 
     /* Shut down the socket for write, which will send a FIN
@@ -510,29 +527,41 @@ int jk_shutdown_socket(jk_sock_t s)
     if (shutdown(s, SHUT_WR)) {
         return jk_close_socket(s);
     }
-#if defined(WIN32)  || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
-                   (const char *) &tmout, sizeof(int)) == 0)
-        rc = 1;
-#elif defined(SO_RCVTIMEO) && defined(USE_SO_RCVTIMEO)
-    tv.tv_sec  = SECONDS_TO_LINGER;
-    tv.tv_usec = 0;
-    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
-                   (const void *) &tv, sizeof(tv)))
-        rc = 1;
+
+    /* Set up to wait for readable data on socket... */
+    FD_ZERO(&rs);
+
+    do {
+        /* Read all data from the peer until we reach "end-of-file"
+         * (FIN from peer) or we've exceeded our overall timeout. If the
+         * backend does not send us bytes within 2 seconds
+         * (a value pulled from Apache 1.3 which seems to work well),
+         * close the connection.
+         */
+        FD_SET(s, &rs);
+        tv.tv_sec  = SECONDS_TO_LINGER;
+        tv.tv_usec = 0;
+
+        if (select((int)s + 1, &rs, NULL, NULL, &tv) > 0) {
+            do {
+#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+                rc = recv(s, &dummy[0], sizeof(dummy), 0);
+                /* Assuming SOCKET_ERROR is -1 on NETWARE too */
+                if (rc == SOCKET_ERROR)
+                    errno = WSAGetLastError() - WSABASEERR;
+#else
+                rc = read(s, &dummy[0], sizeof(dummy));
 #endif
-    /* Read all data from the peer until we reach "end-of-file" (FIN
-     * from peer) or we've exceeded our overall timeout. If the client does
-     * not send us bytes within 12 second, close the connection.
-     */
-    while (rc) {
-        nbytes = jk_tcp_socket_recvfull(s, dummy, sizeof(dummy));
-        if (nbytes <= 0)
+            } while (rc == -1 && (errno == EINTR || errno == EAGAIN));
+
+            if (rc <= 0)
+                break;
+        }
+        else
             break;
-        ttl += SECONDS_TO_LINGER;
-        if (ttl > MAX_SECS_TO_LINGER)
-            break;
-    }
+
+    } while (difftime(time(NULL), start) < MAX_SECS_TO_LINGER);
+
     return jk_close_socket(s);
 }
 
@@ -633,15 +662,19 @@ int jk_is_socket_connected(jk_sock_t sock)
     FD_ZERO(&fd);
     FD_SET(sock, &fd);
 
-    /* Wait one microsecond */
+    /* Initially test the socket without any blocking.
+     */
     tv.tv_sec  = 0;
-    tv.tv_usec = 1;
-    
+    tv.tv_usec = 0;
+
     do {
         rc = select((int)sock + 1, &fd, NULL, NULL, &tv);
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
         errno = WSAGetLastError() - WSABASEERR;
-#endif        
+#endif
+        /* Wait one microsecond on next select, if EINTR */
+        tv.tv_sec  = 0;
+        tv.tv_usec = 1;
     } while (rc == -1 && errno == EINTR);
 
     if (rc == 0) {
@@ -664,7 +697,7 @@ int jk_is_socket_connected(jk_sock_t sock)
         if (ioctl(sock, FIONREAD, (void*)&nr) == 0) {
             return nr == 0 ? 0 : 1;
         }
-#endif        
+#endif
     }
 
     return 0;
