@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -18,7 +19,7 @@
  * Description: In process JNI worker                                      *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Based on:                                                               *
- * Version:     $Revision: 434177 $                                           *
+ * Version:     $Revision: 559197 $                                           *
  ***************************************************************************/
 
 #if !defined(WIN32) && !defined(NETWARE) && !defined(AS400)
@@ -31,6 +32,7 @@
 #include "jk_jni_worker.h"
 #include "jk_util.h"
 
+#if !defined(JNI_VERSION_1_6)
 #if defined LINUX && defined APACHE2_SIGHACK
 #include <pthread.h>
 #include <signal.h>
@@ -202,7 +204,7 @@ static void detach_from_jvm(jni_worker_t * p, jk_logger_t *l);
 
 /*
    Duplicate string and convert it to ASCII on EBDIC based systems
-   Needed for at least AS/400 and BS2000 but what about other EBDIC systems ?
+   Needed for at least AS/400 (before V5R4 ?) and BS2000 but what about other EBDIC systems ?
 */
 static void *strdup_ascii(jk_pool_t *p, char *s)
 {
@@ -248,29 +250,31 @@ static void print_signals(sigset_t * sset)
 
 static int JK_METHOD service(jk_endpoint_t *e,
                              jk_ws_service_t *s,
-                             jk_logger_t *l, int *is_recoverable_error)
+                             jk_logger_t *l, int *is_error)
 {
     jni_endpoint_t *p;
     jint rc;
 
     JK_TRACE_ENTER(l);
 
-    if (is_recoverable_error)
-        *is_recoverable_error = JK_FALSE;
-    if (!e || !e->endpoint_private || !s || !is_recoverable_error) {
+    if (!e || !e->endpoint_private || !s || !is_error) {
         JK_LOG_NULL_PARAMS(l);
+        if (is_error)
+            *is_error = JK_HTTP_SERVER_ERROR;
         JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
     p = e->endpoint_private;
 
+    /* Set returned error to OK */
+    *is_error = JK_HTTP_OK;
+
     if (!p->attached) {
         /* Try to attach */
         if (!(p->env = attach_to_jvm(p->worker, l))) {
             jk_log(l, JK_LOG_EMERG, "Attach failed");
-            /*   Is it recoverable ?? */
-            *is_recoverable_error = JK_TRUE;
+            *is_error = JK_HTTP_SERVER_ERROR;
             JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
@@ -279,7 +283,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
 
     /* we are attached now */
 
-    /* 
+    /*
      * When we call the JVM we cannot know what happens
      * So we can not recover !!!
      */
@@ -305,6 +309,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
     }
     else {
         jk_log(l, JK_LOG_ERROR, "Tomcat FAILED!");
+        *is_error = JK_HTTP_SERVER_ERROR;
         JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
@@ -559,7 +564,7 @@ static int JK_METHOD get_endpoint(jk_worker_t *pThis,
         JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
-    
+
     p = (jni_endpoint_t *) calloc(1, sizeof(jni_endpoint_t));
     if (p) {
         p->attached = JK_FALSE;
@@ -1204,13 +1209,13 @@ static JNIEnv *attach_to_jvm(jni_worker_t *p)
 
 #ifdef LINUX
     linux_signal_hack();
-#endif    
+#endif
 
-    if(0 == (*(p->jvm))->AttachCurrentThread(p->jvm, 
-#ifdef JNI_VERSION_1_2 
+    if(0 == (*(p->jvm))->AttachCurrentThread(p->jvm,
+#ifdef JNI_VERSION_1_2
            (void **)
 #endif
-                                             &rc, 
+                                             &rc,
                                              NULL)) {
         return rc;
     }
@@ -1235,3 +1240,12 @@ static void detach_from_jvm(jni_worker_t * p, jk_logger_t *l)
     }
     JK_TRACE_EXIT(l);
 }
+#else
+int JK_METHOD jni_worker_factory(jk_worker_t **w,
+                                 const char *name, jk_logger_t *l)
+{
+    if (w)
+        *w = NULL;
+    return 0;
+}
+#endif /* JNI_VERSION_1_6 */
