@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -17,7 +18,7 @@
 /***************************************************************************
  * Description: Logger object definitions                                  *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 434087 $                                           *
+ * Version:     $Revision: 559692 $                                           *
  ***************************************************************************/
 
 #ifndef JK_LOGGER_H
@@ -35,13 +36,19 @@ struct jk_logger
 {
     void *logger_private;
     int level;
+    const char *log_fmt;        /* the configured timestamp format for logging */
+    const char *log_fmt_subsec; /* like log_fmt, but milli/micro seconds
+                                   marker replaced, because strftime() doesn't handle those */
+    int    log_fmt_type;        /* do we want milli or microseconds */
+    size_t log_fmt_offset;      /* at which position should we insert */
+    size_t log_fmt_size;        /* how long is this format string */
 
-    int (JK_METHOD * log) (jk_logger_t *l, int level, const char *what);
+    int (JK_METHOD * log) (jk_logger_t *l, int level, int used, char *what);
 
 };
 
-typedef struct file_logger_t file_logger_t;
-struct file_logger_t
+typedef struct jk_file_logger_t jk_file_logger_t;
+struct jk_file_logger_t
 {
     FILE *logfile;
     /* For Apache 2 APR piped logging */
