@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -21,7 +22,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Dan Milstein <danmil@shore.net>                            *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 434345 $                                          *
+ * Version:     $Revision: 548791 $                                          *
  ***************************************************************************/
 
 #ifndef JK_SERVICE_H
@@ -132,11 +133,11 @@ struct jk_ws_service
     const char *server_name;
     unsigned server_port;
     char *server_software;
-    unsigned content_length;        /* integer that represents the content  */
+    jk_uint64_t content_length; /* 64 bit integer that represents the content */
     /* length should be 0 if unknown.        */
     unsigned is_chunked;    /* 1 if content length is unknown (chunked rq) */
     unsigned no_more_chunks;        /* 1 if last chunk has been read */
-    unsigned content_read;  /* number of bytes read */
+    jk_uint64_t content_read;  /* number of bytes read */
 
     /*
      * SSL information
@@ -186,12 +187,12 @@ struct jk_ws_service
     unsigned num_attributes;        /* Number of request attributes     */
 
     /*
-     * The jvm route is in use when the adapter load balance among
-     * several JVMs. It is the ID of a specific JVM in the load balance
-     * group. We are using this variable to implement JVM session
+     * The route is in use when the adapter load balance among
+     * several workers. It is the ID of a specific target in the load balance
+     * group. We are using this variable to implement target session
      * affinity
      */
-    const char *jvm_route;
+    const char *route;
 
     /* Temp solution for auth. For native1 it'll be sent on each request,
        if an option is present. For native2 it'll be sent with the first
@@ -216,6 +217,16 @@ struct jk_ws_service
      */
     int flush_packets;
 
+    /*
+     * If set call flush after AJP13_SEND_HEADERS.
+     */
+    int flush_header;
+
+    /*
+     * HTTP status sent from container.
+     */
+    int http_response_status;
+
     /* Uri worker map. Added for virtual host support
      */
     jk_uri_worker_map_t *uw_map;
@@ -301,8 +312,8 @@ struct jk_ws_service
  */
 struct jk_endpoint
 {
-    size_t rd;
-    size_t wr;
+    jk_uint64_t rd;
+    jk_uint64_t wr;
 
     /*
      * A 'this' pointer which is used by the subclasses of this class to
