@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -19,7 +20,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 440635 $                                          *
+ * Version:     $Revision: 559693 $                                          *
  ***************************************************************************/
 
 
@@ -64,7 +65,10 @@
 #define BALANCE_WORKERS             ("balance_workers")
 #define STICKY_SESSION              ("sticky_session")
 #define STICKY_SESSION_FORCE        ("sticky_session_force")
-#define JVM_ROUTE_OF_WORKER         ("jvm_route")
+#define LOCAL_WORKER_DEPRECATED     ("local_worker")
+#define LOCAL_WORKER_ONLY_DEPRECATED ("local_worker_only")
+#define JVM_ROUTE_OF_WORKER_DEPRECATED ("jvm_route")
+#define ROUTE_OF_WORKER             ("route")
 #define DOMAIN_OF_WORKER            ("domain")
 #define REDIRECT_OF_WORKER          ("redirect")
 #define MOUNT_OF_WORKER             ("mount")
@@ -74,16 +78,30 @@
 #define IS_WORKER_STOPPED_DEPRECATED  ("stopped")
 #define ACTIVATION_OF_WORKER        ("activation")
 #define WORKER_RECOVER_TIME         ("recover_time")
+#define MAX_REPLY_TIMEOUTS_OF_WORKER ("max_reply_timeouts")
 #define WORKER_MAX_PACKET_SIZE      ("max_packet_size")
+#define STYLE_SHEET_OF_WORKER       ("css")
+#define NAMESPACE_OF_WORKER         ("ns")
+#define XML_NAMESPACE_OF_WORKER     ("xmlns")
+#define XML_DOCTYPE_OF_WORKER       ("doctype")
+#define PROP_PREFIX_OF_WORKER       ("prefix")
 
+#define READ_ONLY_OF_WORKER         ("read_only")
+#define USER_OF_WORKER              ("user")
+#define USER_CASE_OF_WORKER         ("user_case_insensitive")
+#define GOOD_RATING_OF_WORKER       ("good")
+#define BAD_RATING_OF_WORKER        ("bad")
 
 #define DEFAULT_WORKER_TYPE         JK_AJP13_WORKER_NAME
 #define SECRET_KEY_OF_WORKER        ("secretkey")
 #define RETRIES_OF_WORKER           ("retries")
+#define STATUS_FAIL_OF_WORKER       ("fail_on_status")
 
 #define DEFAULT_WORKER              JK_AJP13_WORKER_NAME
 #define WORKER_LIST_PROPERTY_NAME     ("worker.list")
+#define LIST_PROPERTY_NAME            ("list")
 #define WORKER_MAINTAIN_PROPERTY_NAME ("worker.maintain")
+#define MAINTAIN_PROPERTY_NAME        ("maintain")
 #define DEFAULT_MAINTAIN_TIME       (60)
 #define DEFAULT_LB_FACTOR           (1)
 #define DEFAULT_DISTANCE            (0)
@@ -95,7 +113,6 @@
 #define TOMCAT50_BRIDGE_NAME        ("tomcat5")
 
 #define HUGE_BUFFER_SIZE (8*1024)
-#define LOG_LINE_SIZE    (1024)
 
 #define MAKE_WORKER_PARAM(P)     \
         strcpy(buf, "worker.");  \
@@ -109,9 +126,17 @@
  * [Mon Mar 26 19:44:48 2001] [jk_uri_worker_map.c (155)]: Into jk_uri_worker_map_t::uri_worker_map_alloc
  * log format used by apache in error.log
  */
-#ifndef JK_TIME_FORMAT
-#define JK_TIME_FORMAT "[%a %b %d %H:%M:%S %Y] "
-#endif
+#define JK_TIME_CONV_MILLI    "%Q"
+#define JK_TIME_CONV_MICRO    "%q"
+#define JK_TIME_PATTERN_MILLI "000"
+#define JK_TIME_PATTERN_MICRO "000000"
+#define JK_TIME_FORMAT_NONE   "[%a %b %d %H:%M:%S %Y] "
+#define JK_TIME_FORMAT_MILLI  "[%a %b %d %H:%M:%S." JK_TIME_CONV_MILLI " %Y] "
+#define JK_TIME_FORMAT_MICRO  "[%a %b %d %H:%M:%S." JK_TIME_CONV_MICRO " %Y] "
+#define JK_TIME_SUBSEC_NONE   (0)
+#define JK_TIME_SUBSEC_MILLI  (1)
+#define JK_TIME_SUBSEC_MICRO  (2)
+#define JK_TIME_MAX_SIZE      (64)
 
 /* Visual C++ Toolkit 2003 support */
 #if defined (_MSC_VER) && (_MSC_VER == 1310)
@@ -122,6 +147,10 @@
 static const char *list_properties[] = {
     BALANCE_WORKERS,
     MOUNT_OF_WORKER,
+    USER_OF_WORKER,
+    GOOD_RATING_OF_WORKER,
+    BAD_RATING_OF_WORKER,
+    STATUS_FAIL_OF_WORKER,
     "list",
     NULL
 };
@@ -147,7 +176,10 @@ static const char *unique_properties[] = {
     LOAD_FACTOR_OF_WORKER,
     STICKY_SESSION,
     STICKY_SESSION_FORCE,
-    JVM_ROUTE_OF_WORKER,
+    LOCAL_WORKER_DEPRECATED,
+    LOCAL_WORKER_ONLY_DEPRECATED,
+    JVM_ROUTE_OF_WORKER_DEPRECATED,
+    ROUTE_OF_WORKER,
     DOMAIN_OF_WORKER,
     REDIRECT_OF_WORKER,
     METHOD_OF_WORKER,
@@ -156,9 +188,17 @@ static const char *unique_properties[] = {
     IS_WORKER_STOPPED_DEPRECATED,
     ACTIVATION_OF_WORKER,
     WORKER_RECOVER_TIME,
+    MAX_REPLY_TIMEOUTS_OF_WORKER,
     WORKER_MAX_PACKET_SIZE,
+    STYLE_SHEET_OF_WORKER,
+    READ_ONLY_OF_WORKER,
     RETRIES_OF_WORKER,
     WORKER_MAINTAIN_PROPERTY_NAME,
+    NAMESPACE_OF_WORKER,
+    XML_NAMESPACE_OF_WORKER,
+    XML_DOCTYPE_OF_WORKER,
+    PROP_PREFIX_OF_WORKER,
+    USER_CASE_OF_WORKER,
     NULL
 };
 
@@ -178,23 +218,118 @@ static const char *deprecated_properties[] = {
     CACHE_TIMEOUT_DEPRECATED,
     RECYCLE_TIMEOUT_DEPRECATED,
     BALANCED_WORKERS_DEPRECATED,
+    JVM_ROUTE_OF_WORKER_DEPRECATED,
+    LOCAL_WORKER_DEPRECATED,
+    LOCAL_WORKER_ONLY_DEPRECATED,
     IS_WORKER_DISABLED_DEPRECATED,
     IS_WORKER_STOPPED_DEPRECATED,
     NULL
 };
 
-/* All entries need to have fixed length 8 chars! */
+static const char *supported_properties[] = {
+    SYSPROPS_OF_WORKER,
+    STDERR_OF_WORKER,
+    STDOUT_OF_WORKER,
+    SECRET_OF_WORKER,
+    MX_OF_WORKER,
+    MS_OF_WORKER,
+    CP_OF_WORKER,
+    BRIDGE_OF_WORKER,
+    JVM_OF_WORKER,
+    LIBPATH_OF_WORKER,
+    CMD_LINE_OF_WORKER,
+    NATIVE_LIB_OF_WORKER,
+    HOST_OF_WORKER,
+    PORT_OF_WORKER,
+    TYPE_OF_WORKER,
+    CACHE_OF_WORKER_DEPRECATED,
+    CACHE_OF_WORKER,
+    CACHE_OF_WORKER_MIN,
+    CACHE_TIMEOUT_DEPRECATED,
+    CACHE_TIMEOUT_OF_WORKER,
+    RECOVERY_OPTS_OF_WORKER,
+    CONNECT_TIMEOUT_OF_WORKER,
+    PREPOST_TIMEOUT_OF_WORKER,
+    REPLY_TIMEOUT_OF_WORKER,
+    SOCKET_TIMEOUT_OF_WORKER,
+    SOCKET_BUFFER_OF_WORKER,
+    SOCKET_KEEPALIVE_OF_WORKER,
+    RECYCLE_TIMEOUT_DEPRECATED,
+    LOAD_FACTOR_OF_WORKER,
+    DISTANCE_OF_WORKER,
+    BALANCED_WORKERS_DEPRECATED,
+    BALANCE_WORKERS,
+    STICKY_SESSION,
+    STICKY_SESSION_FORCE,
+    LOCAL_WORKER_DEPRECATED,
+    LOCAL_WORKER_ONLY_DEPRECATED,
+    JVM_ROUTE_OF_WORKER_DEPRECATED,
+    ROUTE_OF_WORKER,
+    DOMAIN_OF_WORKER,
+    REDIRECT_OF_WORKER,
+    MOUNT_OF_WORKER,
+    METHOD_OF_WORKER,
+    LOCK_OF_WORKER,
+    IS_WORKER_DISABLED_DEPRECATED,
+    IS_WORKER_STOPPED_DEPRECATED,
+    ACTIVATION_OF_WORKER,
+    WORKER_RECOVER_TIME,
+    MAX_REPLY_TIMEOUTS_OF_WORKER,
+    WORKER_MAX_PACKET_SIZE,
+    STYLE_SHEET_OF_WORKER,
+    NAMESPACE_OF_WORKER,
+    XML_NAMESPACE_OF_WORKER,
+    XML_DOCTYPE_OF_WORKER,
+    PROP_PREFIX_OF_WORKER,
+    READ_ONLY_OF_WORKER,
+    USER_OF_WORKER,
+    USER_CASE_OF_WORKER,
+    GOOD_RATING_OF_WORKER,
+    BAD_RATING_OF_WORKER,
+    SECRET_KEY_OF_WORKER,
+    RETRIES_OF_WORKER,
+    STATUS_FAIL_OF_WORKER,
+    LIST_PROPERTY_NAME,
+    MAINTAIN_PROPERTY_NAME
+};
+
 static const char *jk_level_verbs[] = {
     "[" JK_LOG_TRACE_VERB "] ",
     "[" JK_LOG_DEBUG_VERB "] ",
-    "[" JK_LOG_INFO_VERB "]  ",
-    "[" JK_LOG_WARN_VERB "]  ",
+    "[" JK_LOG_INFO_VERB "] ",
+    "[" JK_LOG_WARN_VERB "] ",
     "[" JK_LOG_ERROR_VERB "] ",
     "[" JK_LOG_EMERG_VERB "] ",
     NULL
 };
 
-const char *jk_log_fmt = JK_TIME_FORMAT;
+const char *jk_get_bool(int v)
+{
+    if (v == 0)
+        return "False";
+    else
+        return "True";
+}
+
+int jk_get_bool_code(const char *v, int def)
+{
+    if (!v) {
+        return def;
+    }
+    else if (!strcasecmp(v, "off") ||
+             *v == 'F' || *v == 'f' ||
+             *v == 'N' || *v == 'n' ||
+             *v == '0') {
+        return 0;
+    }
+    else if (!strcasecmp(v, "on") ||
+             *v == 'T' || *v == 't' ||
+             *v == 'Y' || *v == 'y' ||
+             *v == '1') {
+        return 1;
+    }
+    return def;
+}
 
 /* Sleep for 100ms */
 void jk_sleep(int ms)
@@ -215,72 +350,133 @@ void jk_sleep(int ms)
 #endif
 }
 
-static int set_time_str(char *str, int len)
+void jk_set_time_fmt(jk_logger_t *l, const char *jk_log_fmt)
 {
-    time_t t = time(NULL);
-    struct tm *tms;
+    if (l) {
+        char *s;
+        char log_fmt_safe[JK_TIME_MAX_SIZE];
+        char *fmt;
 
-    tms = localtime(&t);
-    return (int)strftime(str, len, jk_log_fmt, tms);
-}
+        if (!jk_log_fmt) {
+#ifndef NO_GETTIMEOFDAY
+            jk_log_fmt = JK_TIME_FORMAT_MILLI;
+#else
+            jk_log_fmt = JK_TIME_FORMAT_NONE;
+#endif
+        }
+        l->log_fmt_type = JK_TIME_SUBSEC_NONE;
+        l->log_fmt_offset = 0;
+        l->log_fmt_size = 0;
+        l->log_fmt_subsec = jk_log_fmt;
+        l->log_fmt = jk_log_fmt;
 
-/* Write at most n characters to the buffer in str, return the
- * number of chars written or -1 if the buffer would have been
- * overflowed.
- *
- * This is portable to any POSIX-compliant system that has /dev/null
- */
-#if !defined(HAVE_VSNPRINTF) && !defined(HAVE_APR)
-static FILE *f = NULL;
-static int vsnprintf(char *str, size_t n, const char *fmt, va_list ap)
-{
-    int res;
+        fmt = (char *)malloc(JK_TIME_MAX_SIZE + strlen(JK_TIME_PATTERN_MICRO));
+        if (fmt) {
+            strncpy(log_fmt_safe, jk_log_fmt, JK_TIME_MAX_SIZE);
+            if ((s = strstr(log_fmt_safe, JK_TIME_CONV_MILLI))) {
+                size_t offset = s - log_fmt_safe;
+                size_t len = strlen(JK_TIME_PATTERN_MILLI);
 
-    if (f == NULL)
-        f = fopen("/dev/null", "w");
-    if (f == NULL)
-        return -1;
+                l->log_fmt_type = JK_TIME_SUBSEC_MILLI;
+                l->log_fmt_offset = offset;
+                strncpy(fmt, log_fmt_safe, offset);
+                strncpy(fmt + offset, JK_TIME_PATTERN_MILLI, len);
+                strncpy(fmt + offset + len,
+                        s + strlen(JK_TIME_CONV_MILLI),
+                        JK_TIME_MAX_SIZE - offset - len);
+                fmt[JK_TIME_MAX_SIZE-1] = '\0';
+                l->log_fmt_subsec = fmt;
+                l->log_fmt_size = strlen(fmt);
+            }
+            else if ((s = strstr(log_fmt_safe, JK_TIME_CONV_MICRO))) {
+                size_t offset = s - log_fmt_safe;
+                size_t len = strlen(JK_TIME_PATTERN_MICRO);
 
-    setvbuf(f, str, _IOFBF, n);
-
-    res = vfprintf(f, fmt, ap);
-
-    if (res > 0 && res < n) {
-        res = vsprintf(str, fmt, ap);
+                l->log_fmt_type = JK_TIME_SUBSEC_MICRO;
+                l->log_fmt_offset = offset;
+                strncpy(fmt, log_fmt_safe, offset);
+                strncpy(fmt + offset, JK_TIME_PATTERN_MICRO, len);
+                strncpy(fmt + offset + len,
+                        s + strlen(JK_TIME_CONV_MICRO),
+                        JK_TIME_MAX_SIZE - offset - len);
+                fmt[JK_TIME_MAX_SIZE-1] = '\0';
+                l->log_fmt_subsec = fmt;
+                l->log_fmt_size = strlen(fmt);
+            }
+        }
     }
-    return res;
 }
-#endif
-#if !defined(HAVE_SNPRINTF) && !defined(HAVE_APR)
-static int snprintf(char *str, size_t n, const char *fmt, ...)
+
+static int set_time_str(char *str, int len, jk_logger_t *l)
 {
-    va_list ap;
-    int res;
+    time_t t;
+    struct tm *tms;
+    int done;
+    char log_fmt[JK_TIME_MAX_SIZE];
 
-    va_start(ap, fmt);
-    res = vsnprintf(str, n, fmt, ap);
-    va_end(ap);
-    return res;
-}
+    if (!l || !l->log_fmt) {
+        return 0;
+    }
+
+    log_fmt[0] = '\0';
+
+#ifndef NO_GETTIMEOFDAY
+    if ( l->log_fmt_type != JK_TIME_SUBSEC_NONE ) {
+        struct timeval tv;
+        int rc = 0;
+
+#ifdef WIN32
+        gettimeofday(&tv, NULL);
+#else
+        rc = gettimeofday(&tv, NULL);
 #endif
+        if (rc == 0) {
+            char subsec[7];
+            t = tv.tv_sec;
+            strncpy(log_fmt, l->log_fmt_subsec, l->log_fmt_size + 1);
+            if (l->log_fmt_type == JK_TIME_SUBSEC_MILLI) {
+                sprintf(subsec, "%03d", (int)(tv.tv_usec/1000));
+                strncpy(log_fmt + l->log_fmt_offset, subsec, 3);
+            }
+            else if (l->log_fmt_type == JK_TIME_SUBSEC_MICRO) {
+                sprintf(subsec, "%06d", (int)(tv.tv_usec));
+                strncpy(log_fmt + l->log_fmt_offset, subsec, 6);
+            }
+        }
+        else {
+            t = time(NULL);
+        }
+    }
+    else {
+        t = time(NULL);
+    }
+#else
+    t = time(NULL);
+#endif
+    tms = localtime(&t);
+    if (log_fmt[0])
+        done = (int)strftime(str, len, log_fmt, tms);
+    else
+        done = (int)strftime(str, len, l->log_fmt, tms);
+    return done;
 
-static int JK_METHOD log_to_file(jk_logger_t *l, int level, const char *what)
+}
+
+static int JK_METHOD log_to_file(jk_logger_t *l, int level, int used, char *what)
 {
     if (l &&
         (l->level <= level || level == JK_LOG_REQUEST_LEVEL) &&
         l->logger_private && what) {
-        size_t sz = strlen(what);
-        if (sz) {
-            file_logger_t *p = l->logger_private;
-            if (fwrite(what, 1, sz, p->logfile)) {
-                /* [V] Flush the dam' thing! */
-                fflush(p->logfile);
-            }
+        jk_file_logger_t *p = l->logger_private;
+        if (p->logfile) {
+            what[used++] = '\n';
+            what[used] = '\0';
+            fputs(what, p->logfile);
+            /* [V] Flush the dam' thing! */
+            fflush(p->logfile);
         }
-
         return JK_TRUE;
     }
-
     return JK_FALSE;
 }
 
@@ -316,19 +512,22 @@ int jk_parse_log_level(const char *level)
 int jk_open_file_logger(jk_logger_t **l, const char *file, int level)
 {
     if (l && file) {
+
         jk_logger_t *rc = (jk_logger_t *)malloc(sizeof(jk_logger_t));
-        file_logger_t *p = (file_logger_t *) malloc(sizeof(file_logger_t));
+        jk_file_logger_t *p = (jk_file_logger_t *) malloc(sizeof(jk_file_logger_t));
         if (rc && p) {
             rc->log = log_to_file;
             rc->level = level;
+            jk_set_time_fmt(rc, NULL);
             rc->logger_private = p;
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
             p->logfile = fopen(file, "a+, o_ccsid=0");
 #else
             p->logfile = fopen(file, "a+");
 #endif
             if (p->logfile) {
                 *l = rc;
+                jk_log(rc, JK_LOG_DEBUG, "log time stamp format is '%s'", rc->log_fmt);
                 return JK_TRUE;
             }
         }
@@ -347,7 +546,7 @@ int jk_open_file_logger(jk_logger_t **l, const char *file, int level)
 int jk_close_file_logger(jk_logger_t **l)
 {
     if (l && *l) {
-        file_logger_t *p = (*l)->logger_private;
+        jk_file_logger_t *p = (*l)->logger_private;
         if (p) {
             fflush(p->logfile);
             fclose(p->logfile);
@@ -366,8 +565,12 @@ int jk_log(jk_logger_t *l,
            const char *fmt, ...)
 {
     int rc = 0;
-    /* Need to reserve space for newline and terminating zero byte. */
-    static int usable_size = HUGE_BUFFER_SIZE-2;
+    /*
+     * Need to reserve space for terminating zero byte
+     * and platform specific line endings added during the call
+     * to the output routing.
+     */
+    static int usable_size = HUGE_BUFFER_SIZE - 3;
     if (!l || !file || !fmt) {
         return -1;
     }
@@ -396,42 +599,53 @@ int jk_log(jk_logger_t *l,
         if (NULL == buf)
             return -1;
 #endif
-        used = set_time_str(buf, usable_size);
+        used = set_time_str(buf, usable_size, l);
 
-        if (line) {
+        if (line) { /* line==0 only used for request log item */
             /* Log [pid:threadid] for all levels except REQUEST. */
             /* This information helps to correlate lines from different logs. */
             /* Performance is no issue, because with production log levels */
             /* we only call it often, if we have a lot of errors */
-#ifdef USE_SPRINTF              /* until we get a snprintf function */
-            rc = sprintf(&buf[used], "[%04d:%04d] ", getpid(),
-                            jk_gettid());
-#else
-            rc = snprintf(&buf[used], usable_size - used,
-                             "[%04d:%04d] ", getpid(), jk_gettid());
-#endif
+            rc = snprintf(buf + used, usable_size - used,
+                          "[%" JK_PID_T_FMT ":%" JK_UINT32_T_FMT "] ", getpid(), jk_gettid());
             used += rc;
-            if (rc < 0 || usable_size - used < 8) {
+            if (rc < 0 ) {
                 return 0;
             }
-            strcat(buf, jk_level_verbs[level]);
-            used += 8;
+
+            rc = (int)strlen(jk_level_verbs[level]);
+            if (usable_size - used >= rc) {
+                strncpy(buf + used, jk_level_verbs[level], rc);
+                used += rc;
+            }
+            else {
+                return 0;           /* [V] not sure what to return... */
+            }
 
             if (funcname) {
-                rc = (int)strlen(funcname) + 2;
-                if (usable_size - used >= rc) {
-                    strcat(buf, funcname);
-                    strcat(buf, "::");
+                rc = (int)strlen(funcname);
+                if (usable_size - used >= rc + 2) {
+                    strncpy(buf + used, funcname, rc);
                     used += rc;
+                    strncpy(buf + used, "::", 2);
+                    used += 2;
+                }
+                else {
+                    return 0;           /* [V] not sure what to return... */
                 }
             }
 
-#ifdef USE_SPRINTF              /* until we get a snprintf function */
-            rc = sprintf(&buf[used], "%s (%d): ", f, line);
-#else
-            rc = snprintf(&buf[used], usable_size - used,
-                             "%s (%d): ", f, line);
-#endif
+            rc = (int)strlen(f);
+            if (usable_size - used >= rc) {
+                strncpy(buf + used, f, rc);
+                used += rc;
+            }
+            else {
+                return 0;           /* [V] not sure what to return... */
+            }
+
+            rc = snprintf(buf + used, usable_size - used,
+                          " (%d): ", line);
             used += rc;
             if (rc < 0 || usable_size - used < 0) {
                 return 0;           /* [V] not sure what to return... */
@@ -439,20 +653,15 @@ int jk_log(jk_logger_t *l,
         }
 
         va_start(args, fmt);
-#ifdef USE_VSPRINTF             /* until we get a vsnprintf function */
-        rc = vsprintf(buf + used, fmt, args);
-#else
         rc = vsnprintf(buf + used, usable_size - used, fmt, args);
-#endif
+        va_end(args);
         if ( rc <= usable_size - used ) {
             used += rc;
         } else {
             used = usable_size;
         }
-        buf[used] = '\n';
-        buf[used+1] = 0;
-        va_end(args);
-        l->log(l, level, buf);
+        l->log(l, level, used, buf);
+
 #ifdef NETWARE
         free(buf);
 #endif
@@ -472,13 +681,20 @@ const char *jk_get_worker_type(jk_map_t *m, const char *wname)
     return jk_map_get_string(m, buf, DEFAULT_WORKER_TYPE);
 }
 
-const char *jk_get_worker_jvm_route(jk_map_t *m, const char *wname, const char *def)
+const char *jk_get_worker_route(jk_map_t *m, const char *wname, const char *def)
 {
     char buf[1024];
+    const char *v;
     if (!m || !wname) {
         return NULL;
     }
-    MAKE_WORKER_PARAM(JVM_ROUTE_OF_WORKER);
+    MAKE_WORKER_PARAM(ROUTE_OF_WORKER);
+    v = jk_map_get_string(m, buf, NULL);
+    if (v) {
+        return v;
+    }
+    /* Try old jvm_route directive */
+    MAKE_WORKER_PARAM(JVM_ROUTE_OF_WORKER_DEPRECATED);
     return jk_map_get_string(m, buf, def);
 }
 
@@ -648,6 +864,19 @@ int jk_get_worker_recover_timeout(jk_map_t *m, const char *wname, int def)
     return jk_map_get_int(m, buf, def);
 }
 
+int jk_get_worker_max_reply_timeouts(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(MAX_REPLY_TIMEOUTS_OF_WORKER);
+
+    return jk_map_get_int(m, buf, def);
+}
+
 int jk_get_worker_socket_buffer(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -780,19 +1009,19 @@ const char *jk_get_worker_secret_key(jk_map_t *m, const char *wname)
     return jk_map_get_string(m, buf, NULL);
 }
 
-int jk_get_worker_list(jk_map_t *m, char ***list, unsigned *num_of_wokers)
+int jk_get_worker_list(jk_map_t *m, char ***list, unsigned *num_of_workers)
 {
-    if (m && list && num_of_wokers) {
+    if (m && list && num_of_workers) {
         char **ar = jk_map_get_string_list(m,
                                         WORKER_LIST_PROPERTY_NAME,
-                                        num_of_wokers,
+                                        num_of_workers,
                                         DEFAULT_WORKER);
         if (ar) {
             *list = ar;
             return JK_TRUE;
         }
         *list = NULL;
-        *num_of_wokers = 0;
+        *num_of_workers = 0;
     }
 
     return JK_FALSE;
@@ -837,26 +1066,14 @@ int jk_get_worker_activation(jk_map_t *m, const char *wname)
     MAKE_WORKER_PARAM(ACTIVATION_OF_WORKER);
     v = jk_map_get_string(m, buf, NULL);
     if (v) {
-        if (*v == 'a' || *v == 'A')
-            return JK_LB_ACTIVATION_ACTIVE;
-        else if (*v == 's' || *v == 'S')
-            return JK_LB_ACTIVATION_STOPPED;
-        else if (*v == 'd' || *v == 'D')
-            return JK_LB_ACTIVATION_DISABLED;
-        else
-            return JK_LB_ACTIVATION_ACTIVE;
+        return jk_lb_get_activation_code(v);
     }
     else if (jk_get_is_worker_stopped(m, wname))
         return JK_LB_ACTIVATION_STOPPED;
     else if (jk_get_is_worker_disabled(m, wname))
         return JK_LB_ACTIVATION_DISABLED;
     else
-        return JK_LB_ACTIVATION_ACTIVE;
-}
-
-void jk_set_log_format(const char *logformat)
-{
-    jk_log_fmt = (logformat) ? logformat : JK_TIME_FORMAT;
+        return JK_LB_ACTIVATION_DEF;
 }
 
 int jk_get_lb_factor(jk_map_t *m, const char *wname)
@@ -923,16 +1140,7 @@ int jk_get_lb_method(jk_map_t *m, const char *wname)
 
     MAKE_WORKER_PARAM(METHOD_OF_WORKER);
     v = jk_map_get_string(m, buf, NULL);
-    if (!v)
-        return JK_LB_METHOD_DEF;
-    else if  (*v == 't' || *v == 'T' || *v == '1')
-        return JK_LB_METHOD_TRAFFIC;
-    else if  (*v == 'r' || *v == 'R' || *v == '0')
-        return JK_LB_METHOD_REQUESTS;
-    else if  (*v == 'b' || *v == 'B' || *v == '2')
-        return JK_LB_METHOD_BUSYNESS;
-    else
-        return JK_LB_METHOD_DEF;
+    return jk_lb_get_method_code(v);
 }
 
 int jk_get_lb_lock(jk_map_t *m, const char *wname)
@@ -945,14 +1153,7 @@ int jk_get_lb_lock(jk_map_t *m, const char *wname)
 
     MAKE_WORKER_PARAM(LOCK_OF_WORKER);
     v = jk_map_get_string(m, buf, NULL);
-    if (!v)
-        return JK_LB_LOCK_DEF;
-    else if  (*v == 'o' || *v == 'O' || *v == '0')
-        return JK_LB_LOCK_OPTIMISTIC;
-    else if  (*v == 'p' || *v == 'P' || *v == '1')
-        return JK_LB_LOCK_PESSIMISTIC;
-    else
-        return JK_LB_LOCK_DEF;
+    return jk_lb_get_lock_code(v);
 }
 
 int jk_get_max_packet_size(jk_map_t *m, const char *wname)
@@ -964,41 +1165,216 @@ int jk_get_max_packet_size(jk_map_t *m, const char *wname)
         return DEF_BUFFER_SZ;
     }
 
-    MAKE_WORKER_PARAM(DISTANCE_OF_WORKER);
+    MAKE_WORKER_PARAM(WORKER_MAX_PACKET_SIZE);
     sz = jk_map_get_int(m, buf, DEF_BUFFER_SZ);
     sz = JK_ALIGN(sz, 1024);
     if (sz < DEF_BUFFER_SZ)
         sz = DEF_BUFFER_SZ;
     else if (sz > 64*1024)
         sz = 64*1024;
-    
+
     return sz;
 }
 
+int jk_get_worker_fail_on_status(jk_map_t *m, const char *wname,
+                                 int *list, unsigned int list_size)
+{
+    char buf[1024];
+    if (!m || !wname || !list) {
+        return 0;
+    }
+    MAKE_WORKER_PARAM(STATUS_FAIL_OF_WORKER);
+    if (list_size) {
+        return jk_map_get_int_list(m, buf,
+                                   list, list_size,
+                                   NULL);
+    }
+
+    return 0;
+}
+
+int jk_get_worker_user_case_insensitive(jk_map_t *m, const char *wname)
+{
+    int rc = JK_FALSE;
+    char buf[1024];
+    if (m && wname) {
+        int value;
+        MAKE_WORKER_PARAM(USER_CASE_OF_WORKER);
+        value = jk_map_get_bool(m, buf, 0);
+        if (value)
+            rc = JK_TRUE;
+    }
+    return rc;
+
+}
+
+const char *jk_get_worker_style_sheet(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return NULL;
+    }
+
+    MAKE_WORKER_PARAM(STYLE_SHEET_OF_WORKER);
+
+    return jk_map_get_string(m, buf, def);
+}
+
+const char *jk_get_worker_name_space(jk_map_t *m, const char *wname, const char *def)
+{
+    const char *rc;
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    MAKE_WORKER_PARAM(NAMESPACE_OF_WORKER);
+    rc = jk_map_get_string(m, buf, def);
+    if (*rc == '-')
+        return "";
+    else
+        return rc;
+}
+
+const char *jk_get_worker_xmlns(jk_map_t *m, const char *wname, const char *def)
+{
+    const char *rc;
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    MAKE_WORKER_PARAM(XML_NAMESPACE_OF_WORKER);
+    rc = jk_map_get_string(m, buf, def);
+    if (*rc == '-')
+        return "";
+    else
+        return rc;
+}
+
+const char *jk_get_worker_xml_doctype(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    MAKE_WORKER_PARAM(XML_DOCTYPE_OF_WORKER);
+    return jk_map_get_string(m, buf, def);
+}
+
+const char *jk_get_worker_prop_prefix(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    MAKE_WORKER_PARAM(PROP_PREFIX_OF_WORKER);
+    return jk_map_get_string(m, buf, def);
+}
+
+int jk_get_is_read_only(jk_map_t *m, const char *wname)
+{
+    int rc = JK_FALSE;
+    char buf[1024];
+    if (m && wname) {
+        int value;
+        MAKE_WORKER_PARAM(READ_ONLY_OF_WORKER);
+        value = jk_map_get_bool(m, buf, 0);
+        if (value)
+            rc = JK_TRUE;
+    }
+    return rc;
+}
+
+int jk_get_worker_user_list(jk_map_t *m,
+                            const char *wname,
+                            char ***list, unsigned int *num)
+{
+    char buf[1024];
+
+    if (m && list && num && wname) {
+        char **ar = NULL;
+
+        MAKE_WORKER_PARAM(USER_OF_WORKER);
+        ar = jk_map_get_string_list(m, buf, num, NULL);
+        if (ar) {
+            *list = ar;
+            return JK_TRUE;
+        }
+        *list = NULL;
+        *num = 0;
+    }
+
+    return JK_FALSE;
+}
+
+int jk_get_worker_good_rating(jk_map_t *m,
+                              const char *wname,
+                              char ***list, unsigned int *num)
+{
+    char buf[1024];
+
+    if (m && list && num && wname) {
+        char **ar = NULL;
+
+        MAKE_WORKER_PARAM(GOOD_RATING_OF_WORKER);
+        ar = jk_map_get_string_list(m, buf, num, NULL);
+        if (ar) {
+            *list = ar;
+            return JK_TRUE;
+        }
+        *list = NULL;
+        *num = 0;
+    }
+
+    return JK_FALSE;
+}
+
+int jk_get_worker_bad_rating(jk_map_t *m,
+                             const char *wname,
+                             char ***list, unsigned int *num)
+{
+    char buf[1024];
+
+    if (m && list && num && wname) {
+        char **ar = NULL;
+
+        MAKE_WORKER_PARAM(BAD_RATING_OF_WORKER);
+        ar = jk_map_get_string_list(m, buf, num, NULL);
+        if (ar) {
+            *list = ar;
+            return JK_TRUE;
+        }
+        *list = NULL;
+        *num = 0;
+    }
+
+    return JK_FALSE;
+}
+
 int jk_get_lb_worker_list(jk_map_t *m,
                           const char *wname,
-                          char ***list, unsigned int *num_of_wokers)
+                          char ***list, unsigned int *num_of_workers)
 {
     char buf[1024];
 
-    if (m && list && num_of_wokers && wname) {
+    if (m && list && num_of_workers && wname) {
         char **ar = NULL;
 
         MAKE_WORKER_PARAM(BALANCE_WORKERS);
-        ar = jk_map_get_string_list(m, buf, num_of_wokers, NULL);
+        ar = jk_map_get_string_list(m, buf, num_of_workers, NULL);
         if (ar) {
             *list = ar;
             return JK_TRUE;
         }
         /* Try old balanced_workers directive */
         MAKE_WORKER_PARAM(BALANCED_WORKERS_DEPRECATED);
-        ar = jk_map_get_string_list(m, buf, num_of_wokers, NULL);
+        ar = jk_map_get_string_list(m, buf, num_of_workers, NULL);
         if (ar) {
             *list = ar;
             return JK_TRUE;
         }
         *list = NULL;
-        *num_of_wokers = 0;
+        *num_of_workers = 0;
     }
 
     return JK_FALSE;
@@ -1163,18 +1539,36 @@ int jk_get_worker_cmd_line(jk_map_t *m, const char *wname, const char **cmd_line
 }
 
 
+int jk_stat(const char *f, struct stat * statbuf)
+{
+  int rc;
+/**
+ * i5/OS V5R4 expect filename in ASCII for fopen but required them in EBCDIC for stat()
+ */
+#ifdef AS400_UTF8
+  char *ptr;
+
+  ptr = (char *)malloc(strlen(f) + 1);
+  jk_ascii2ebcdic((char *)f, ptr);
+  rc = stat(ptr, statbuf);
+  free(ptr);
+#else
+  rc = stat(f, statbuf);
+#endif
+
+  return (rc);
+}
+
+
 int jk_file_exists(const char *f)
 {
     if (f) {
         struct stat st;
-#ifdef AS400
-        if ((0 == stat(f, &st)) && (st.st_mode & _S_IFREG)) {
-#else
-        if ((0 == stat(f, &st)) && (st.st_mode & S_IFREG)) {
-#endif
-            return JK_TRUE;
-        }
+
+        if ((0 == jk_stat(f, &st)) && (st.st_mode & S_IFREG))
+      return JK_TRUE;
     }
+
     return JK_FALSE;
 }
 
@@ -1242,6 +1636,24 @@ int jk_is_deprecated_property(const char *prp_name)
     }
     return JK_FALSE;
 }
+/*
+ * Check that property is a valid one (to prevent user typos).
+ * Only property starting with worker.
+ */
+int jk_is_valid_property(const char *prp_name)
+{
+    const char **props;
+    if (memcmp(prp_name, "worker.", 7))
+        return JK_TRUE;
+
+    props = &supported_properties[0];
+    while (*props) {
+        if (jk_is_some_property(prp_name, *props, "."))
+            return JK_TRUE;
+        props++;
+    }
+    return JK_FALSE;
+}
 
 int jk_get_worker_stdout(jk_map_t *m, const char *wname, const char **stdout_name)
 {
@@ -1310,7 +1722,7 @@ int jk_get_worker_libpath(jk_map_t *m, const char *wname, const char **libpath)
 char **jk_parse_sysprops(jk_pool_t *p, const char *sysprops)
 {
     char **rc = NULL;
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
     char *lasts;
 #endif
 
@@ -1328,7 +1740,7 @@ char **jk_parse_sysprops(jk_pool_t *p, const char *sysprops)
             rc = jk_pool_alloc(p, (num_of_prps + 1) * sizeof(char *));
             if (rc) {
                 unsigned i = 0;
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
                 char *tmp = strtok_r(prps, "*", &lasts);
 #else
                 char *tmp = strtok(prps, "*");
@@ -1336,7 +1748,7 @@ char **jk_parse_sysprops(jk_pool_t *p, const char *sysprops)
 
                 while (tmp && i < num_of_prps) {
                     rc[i] = tmp;
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
                     tmp = strtok_r(NULL, "*", &lasts);
 #else
                     tmp = strtok(NULL, "*");
@@ -1407,23 +1819,140 @@ void jk_init_ws_service(jk_ws_service_t *s)
     s->attributes_names = NULL;
     s->attributes_values = NULL;
     s->num_attributes = 0;
-    s->jvm_route = NULL;
+    s->route = NULL;
     s->retries = JK_RETRIES;
     s->add_log_items = NULL;
 }
 
 #ifdef _MT_CODE_PTHREAD
-int jk_gettid()
+jk_uint32_t jk_gettid()
 {
-    pthread_t t = pthread_self();
+    union {
+        pthread_t tid;
+        jk_uint64_t alignme;
+    } u;
 #ifdef AS400
-    /* OS400 use 64 bits ThreadId, get only low 32 bits for now */
+    /* OS400 use 64 bits ThreadId */
     pthread_id_np_t       tid;
-    pthread_getunique_np(&t, &tid);
-    return ((int)(tid.intId.lo & 0xFFFFFFFF));
+#endif /* AS400 */
+    u.tid = pthread_self();
+#ifdef AS400
+    /* Get only low 32 bits for now */
+    pthread_getunique_np(&(u.tid), &tid);
+    return ((jk_uint32_t)(tid.intId.lo & 0xFFFFFFFF));
 #else
-    int tid = (int)(t & 0xFFFF);
-    return tid;
+    switch(sizeof(pthread_t)) {
+    case sizeof(jk_uint32_t):
+        return *(jk_uint32_t *)&u.tid;
+    case sizeof(jk_uint64_t):
+        return (*(jk_uint64_t *)&u.tid) & 0xFFFFFFFF;
+    default:
+        return 0;
+    }
 #endif /* AS400 */
 }
 #endif
+
+/***
+ * ASCII <-> EBCDIC conversions
+ *
+ * For now usefull only in i5/OS V5R4 where UTF and EBCDIC mode are mixed
+ */
+
+#ifdef AS400_UTF8
+
+/* EBCDIC to ASCII translation table */
+static u_char ebcdic_to_ascii[256] =
+{
+  0x00,0x01,0x02,0x03,0x20,0x09,0x20,0x7f, /* 00-07 */
+  0x20,0x20,0x20,0x0b,0x0c,0x0d,0x0e,0x0f, /* 08-0f */
+  0x10,0x11,0x12,0x13,0x20,0x0a,0x08,0x20, /* 10-17 */
+  0x18,0x19,0x20,0x20,0x20,0x1d,0x1e,0x1f, /* 18-1f */
+  0x20,0x20,0x1c,0x20,0x20,0x0a,0x17,0x1b, /* 20-27 */
+  0x20,0x20,0x20,0x20,0x20,0x05,0x06,0x07, /* 28-2f */
+  0x20,0x20,0x16,0x20,0x20,0x20,0x20,0x04, /* 30-37 */
+  0x20,0x20,0x20,0x20,0x14,0x15,0x20,0x1a, /* 38-3f */
+  0x20,0x20,0x83,0x84,0x85,0xa0,0xc6,0x86, /* 40-47 */
+  0x87,0xa4,0xbd,0x2e,0x3c,0x28,0x2b,0x7c, /* 48-4f */
+  0x26,0x82,0x88,0x89,0x8a,0xa1,0x8c,0x8b, /* 50-57 */
+  0x8d,0xe1,0x21,0x24,0x2a,0x29,0x3b,0xaa, /* 58-5f */
+  0x2d,0x2f,0xb6,0x8e,0xb7,0xb5,0xc7,0x8f, /* 60-67 */
+  0x80,0xa5,0xdd,0x2c,0x25,0x5f,0x3e,0x3f, /* 68-6f */
+  0x9b,0x90,0xd2,0xd3,0xd4,0xd6,0xd7,0xd8, /* 70-77 */
+  0xde,0x60,0x3a,0x23,0x40,0x27,0x3d,0x22, /* 78-7f */
+  0x9d,0x61,0x62,0x63,0x64,0x65,0x66,0x67, /* 80-87 */
+  0x68,0x69,0xae,0xaf,0xd0,0xec,0xe7,0xf1, /* 88-8f */
+  0xf8,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,0x70, /* 90-97 */
+  0x71,0x72,0xa6,0xa7,0x91,0xf7,0x92,0xcf, /* 98-9f */
+  0xe6,0x7e,0x73,0x74,0x75,0x76,0x77,0x78, /* a8-a7 */
+  0x79,0x7a,0xad,0xa8,0xd1,0xed,0xe8,0xa9, /* a8-af */
+  0x5e,0x9c,0xbe,0xfa,0xb8,0x15,0x14,0xac, /* b0-b7 */
+  0xab,0xf3,0x5b,0x5d,0xee,0xf9,0xef,0x9e, /* b8-bf */
+  0x7b,0x41,0x42,0x43,0x44,0x45,0x46,0x47, /* c0-c7 */
+  0x48,0x49,0xf0,0x93,0x94,0x95,0xa2,0xe4, /* c8-cf */
+  0x7d,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,0x50, /* d0-d7 */
+  0x51,0x52,0xfb,0x96,0x81,0x97,0xa3,0x98, /* d8-df */
+  0x5c,0xf6,0x53,0x54,0x55,0x56,0x57,0x58, /* e0-e7 */
+  0x59,0x5a,0xfc,0xe2,0x99,0xe3,0xe0,0xe5, /* e8-ef */
+  0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37, /* f0-f7 */
+  0x38,0x39,0xfd,0xea,0x9a,0xeb,0xe9,0xff  /* f8-ff */
+};
+
+/* ASCII to EBCDIC translation table */
+static u_char ascii_to_ebcdic[256] =
+{
+  0x00,0x01,0x02,0x03,0x37,0x2d,0x2e,0x2f, /* 00-07 */
+  0x16,0x05,0x25,0x0b,0x0c,0x0d,0x0e,0x0f, /* 08-0f */
+  0x10,0x11,0x12,0x13,0x3c,0x3d,0x32,0x26, /* 10-17 */
+  0x18,0x19,0x3f,0x27,0x22,0x1d,0x1e,0x1f, /* 18-1f */
+  0x40,0x5a,0x7f,0x7b,0x5b,0x6c,0x50,0x7d, /* 20-27 */
+  0x4d,0x5d,0x5c,0x4e,0x6b,0x60,0x4b,0x61, /* 28-2f */
+  0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7, /* 30-37 */
+  0xf8,0xf9,0x7a,0x5e,0x4c,0x7e,0x6e,0x6f, /* 38-3f */
+  0x7c,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,0xc7, /* 40-47 */
+  0xc8,0xc9,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6, /* 48-4f */
+  0xd7,0xd8,0xd9,0xe2,0xe3,0xe4,0xe5,0xe6, /* 50-57 */
+  0xe7,0xe8,0xe9,0xba,0xe0,0xbb,0xb0,0x6d, /* 58-5f */
+  0x79,0x81,0x82,0x83,0x84,0x85,0x86,0x87, /* 60-67 */
+  0x88,0x89,0x91,0x92,0x93,0x94,0x95,0x96, /* 68-6f */
+  0x97,0x98,0x99,0xa2,0xa3,0xa4,0xa5,0xa6, /* 70-77 */
+  0xa7,0xa8,0xa9,0xc0,0x4f,0xd0,0xa1,0x07, /* 78-7f */
+  0x68,0xdc,0x51,0x42,0x43,0x44,0x47,0x48, /* 80-87 */
+  0x52,0x53,0x54,0x57,0x56,0x58,0x63,0x67, /* 88-8f */
+  0x71,0x9c,0x9e,0xcb,0xcc,0xcd,0xdb,0xdd, /* 90-97 */
+  0xdf,0xec,0xfc,0x70,0xb1,0x80,0xbf,0x40, /* 98-9f */
+  0x45,0x55,0xee,0xde,0x49,0x69,0x9a,0x9b, /* a8-a7 */
+  0xab,0xaf,0x5f,0xb8,0xb7,0xaa,0x8a,0x8b, /* a8-af */
+  0x40,0x40,0x40,0x40,0x40,0x65,0x62,0x64, /* b0-b7 */
+  0xb4,0x40,0x40,0x40,0x40,0x4a,0xb2,0x40, /* b8-bf */
+  0x40,0x40,0x40,0x40,0x40,0x40,0x46,0x66, /* c0-c7 */
+  0x40,0x40,0x40,0x40,0x40,0x40,0x40,0x9f, /* c8-cf */
+  0x8c,0xac,0x72,0x73,0x74,0x89,0x75,0x76, /* d0-d7 */
+  0x77,0x40,0x40,0x40,0x40,0x6a,0x78,0x40, /* d8-df */
+  0xee,0x59,0xeb,0xed,0xcf,0xef,0xa0,0x8e, /* e0-e7 */
+  0xae,0xfe,0xfb,0xfd,0x8d,0xad,0xbc,0xbe, /* e8-ef */
+  0xca,0x8f,0x40,0xb9,0xb6,0xb5,0xe1,0x9d, /* f0-f7 */
+  0x90,0xbd,0xb3,0xda,0xea,0xfa,0x40,0x40  /* f8-ff */
+};
+
+void jk_ascii2ebcdic(char *src, char *dst) {
+    char c;
+
+    while ((c = *src++) != 0) {
+        *dst++ = ascii_to_ebcdic[(unsigned int)c];
+    }
+
+    *dst = 0;
+}
+
+void jk_ebcdic2ascii(char *src, char *dst) {
+    char c;
+
+    while ((c = *src++) != 0) {
+        *dst++ = ebcdic_to_ascii[(unsigned int)c];
+    }
+
+    *dst = 0;
+}
+
+#endif
