@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -18,7 +19,7 @@
  * Description: NT System service for Tomcat                       *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  *              Dave Oxley <Dave@JungleMoss.com>                           *
- * Version:     $Revision: 437379 $                                           *
+ * Version:     $Revision: 550203 $                                           *
  ***************************************************************************/
 
 #include "jk_global.h"
@@ -381,7 +382,7 @@ void install_service(char *name,
         if (dst >= szTrueName + sizeof(szTrueName) - 1) {
             break;
         }
-        if (!isspace(*src) && *src != '/' && *src != '\\') {
+        if (!isspace((int)(*src)) && *src != '/' && *src != '\\') {
             *(dst++) = *src;
         }
     }
@@ -955,7 +956,8 @@ static int exec_cmd(const char *name, HANDLE *hTomcat, char *cmdLine)
             jk_map_t *init_map;
             
             if(jk_map_alloc(&init_map)) {
-                if(jk_map_read_properties(init_map, prp_file, NULL, NULL)) {
+                if(jk_map_read_properties(init_map, prp_file, NULL,
+                                          JK_MAP_HANDLE_DUPLICATES, NULL)) {
                     jk_tomcat_startup_data_t data;
                     jk_pool_t p;
                     jk_pool_atom_t buf[HUGE_POOL_SIZE];
@@ -1000,8 +1002,8 @@ static int exec_cmd(const char *name, HANDLE *hTomcat, char *cmdLine)
 
                         memset(&processInformation, 0, sizeof(processInformation));
                         
-						if( cmdLine==NULL ) 
-							cmdLine=data.cmd_line;
+                        if( cmdLine==NULL ) 
+                            cmdLine=data.cmd_line;
 
                         printf(cmdLine);
                         if(CreateProcess(data.java_bin,
@@ -1023,7 +1025,7 @@ static int exec_cmd(const char *name, HANDLE *hTomcat, char *cmdLine)
                             shutdown_port = data.shutdown_port;
                             shutdown_secret = data.shutdown_secret;
                             shutdown_protocol = strdup(data.shutdown_protocol);
-							shutdown_cmd = strdup(data.stop_cmd);
+                            shutdown_cmd = strdup(data.stop_cmd);
 
                             return JK_TRUE;
                         } else {
