@@ -1,15 +1,12 @@
 package org.apache.tomcat.jni;
 
+import java.io.InputStream;
 import java.util.Properties;
 
-import java.io.*;
-import java.net.*;
-import java.lang.*;
-
 /** Local Socket server example
  *
  * @author Mladen Turk
- * @version $Revision: 300883 $, $Date: 2005-06-24 03:22:58 -0500 (Fri, 24 Jun 2005) $
+ * @version $Revision: 473804 $, $Date: 2006-11-11 14:56:15 -0700 (Sat, 11 Nov 2006) $
  */
 
 public class LocalServer {
@@ -41,7 +38,6 @@ public class LocalServer {
 
     public LocalServer()
     {
-        int i;
         serverPool = Pool.create(0);
         try {
             serverAcceptor = new Acceptor();
@@ -65,7 +61,7 @@ public class LocalServer {
     }
 
     /* Acceptor thread. Listens for new connections */
-    private class Acceptor extends Thread {
+    private class Acceptor extends java.lang.Thread {
         private long serverSock = 0;
         private long inetAddress = 0;
         private long pool = 0;
@@ -107,7 +103,7 @@ public class LocalServer {
         }
     }
 
-    private class Worker extends Thread {
+    private class Worker extends java.lang.Thread {
         private int workerId = 0;
         private long clientSock = 0;
         private byte [] wellcomeMsg = null;
