@@ -1,15 +1,12 @@
 package org.apache.tomcat.jni;
 
+import java.io.InputStream;
 import java.util.Properties;
 
-import java.io.*;
-import java.net.*;
-import java.lang.*;
-
 /** SSL Server server example
  *
  * @author Mladen Turk
- * @version $Revision: 300872 $, $Date: 2005-06-21 11:20:45 -0500 (Tue, 21 Jun 2005) $
+ * @version $Revision: 473804 $, $Date: 2006-11-11 14:56:15 -0700 (Sat, 11 Nov 2006) $
  */
 
 public class SSLServer {
@@ -54,7 +51,6 @@ public class SSLServer {
 
     public SSLServer()
     {
-        int i;
         serverPool = Pool.create(0);
         try {
             /* Create SSL Context, one for each Virtual Host */
@@ -85,7 +81,7 @@ public class SSLServer {
     }
 
     /* Acceptor thread. Listens for new connections */
-    private class Acceptor extends Thread {
+    private class Acceptor extends java.lang.Thread {
         private long serverSock = 0;
         private long inetAddress = 0;
         private long pool = 0;
@@ -164,7 +160,7 @@ public class SSLServer {
         }
     }
 
-    private class Worker extends Thread {
+    private class Worker extends java.lang.Thread {
         private int workerId = 0;
         private long clientSock = 0;
         private byte [] wellcomeMsg = null;
