@@ -1,8 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -16,7 +17,7 @@
 /** SSL Utilities
  *
  * @author Mladen Turk
- * @version $Revision: 439960 $, $Date: 2006-09-04 02:15:58 -0500 (Mon, 04 Sep 2006) $
+ * @version $Revision: 479112 $, $Date: 2006-11-25 03:22:33 -0700 (Sat, 25 Nov 2006) $
  */
 
 #include "tcn.h"
@@ -80,8 +81,9 @@ int SSL_password_prompt(tcn_pass_cb_t *data)
     if (data->cb.obj) {
         JNIEnv *e;
         jobject  o;
-        jstring  prompt = AJP_TO_JSTRING(data->prompt);
+        jstring  prompt;
         tcn_get_java_env(&e);
+        prompt = AJP_TO_JSTRING(data->prompt);
         if ((o = (*e)->CallObjectMethod(e, data->cb.obj,
                             data->cb.mid[0], prompt))) {
             TCN_ALLOC_CSTRING(o);
@@ -670,10 +672,4 @@ int SSL_callback_SSL_verify(int ok, X509_STORE_CTX *ctx)
     return ok;
 }
 
-#else
-/* OpenSSL is not supported
- * If someday we make OpenSSL optional
- * APR_ENOTIMPL will go here
- */
-#error "No OpenSSL Toolkit defined."
 #endif
