@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -49,7 +50,7 @@ import org.apache.tomcat.util.http.MimeHeaders;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 320670 $ $Date: 2005-10-13 00:39:55 -0500 (Thu, 13 Oct 2005) $
+ * @version $Revision: 536382 $ $Date: 2007-05-09 02:05:00 +0200 (Wed, 09 May 2007) $
  */
 
 public class FormAuthenticator
@@ -367,7 +368,7 @@ public class FormAuthenticator
       String requestURI = request.getRequestURI();
       if (requestURI == null)
           return (false);
-      return (requestURI.equals(request.getRequestURI()));
+      return (requestURI.equals(sreq.getRequestURI()));
 
     }
 
@@ -427,7 +428,14 @@ public class FormAuthenticator
     
                 // Set content type
                 MessageBytes contentType = MessageBytes.newInstance();
-                contentType.setString("application/x-www-form-urlencoded");
+                
+                // If no content type specified, use default for POST
+                String savedContentType = saved.getContentType();
+                if (savedContentType == null) {
+                    savedContentType = "application/x-www-form-urlencoded";
+                }
+
+                contentType.setString(savedContentType);
                 request.getCoyoteRequest().setContentType(contentType);
             }
         }
@@ -487,6 +495,7 @@ public class FormAuthenticator
                 body.append(buffer, 0, bytesRead);
             }
             saved.setBody(body);
+            saved.setContentType(request.getContentType());
         }
 
         saved.setMethod(request.getMethod());
