@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -41,7 +42,7 @@ import org.apache.tomcat.util.http.ServerCookie;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 426355 $ $Date: 2006-07-27 20:32:57 -0500 (Thu, 27 Jul 2006) $
+ * @version $Revision: 523135 $ $Date: 2007-03-28 03:36:22 +0200 (Wed, 28 Mar 2007) $
  */
 
 public class CoyoteAdapter
@@ -49,6 +50,8 @@ public class CoyoteAdapter
  {
     private static Log log = LogFactory.getLog(CoyoteAdapter.class);
 
+    protected static final boolean ALLOW_BACKSLASH = 
+        Boolean.valueOf(System.getProperty("org.apache.catalina.connector.CoyoteAdapter.ALLOW_BACKSLASH", "false")).booleanValue();
     // -------------------------------------------------------------- Constants
 
 
@@ -140,7 +143,7 @@ public class CoyoteAdapter
         }
 
         try {
-
+            req.getRequestProcessor().setWorkerThreadName(Thread.currentThread().getName());
             // Parse and set Catalina and configuration specific 
             // request parameters
             if ( postParseRequest(req, request, res, response) ) {
@@ -156,6 +159,7 @@ public class CoyoteAdapter
         } catch (Throwable t) {
             log.error(sm.getString("coyoteAdapter.service"), t);
         } finally {
+            req.getRequestProcessor().setWorkerThreadName(null);
             // Recycle the wrapper request and response
             request.recycle();
             response.recycle();
@@ -225,8 +229,8 @@ public class CoyoteAdapter
                 req.getURLDecoder().convert(decodedURI, false);
             } catch (IOException ioe) {
                 res.setStatus(400);
-                res.setMessage("Invalid URI");
-                throw ioe;
+                res.setMessage("Invalid URI: " + ioe.getMessage());
+                return false;
             }
             // Normalization
             if (!normalize(req.decodedURI())) {
@@ -514,8 +518,13 @@ public class CoyoteAdapter
         // Replace '\' with '/'
         // Check for null byte
         for (pos = start; pos < end; pos++) {
-            if (b[pos] == (byte) '\\')
-                b[pos] = (byte) '/';
+            if (b[pos] == (byte) '\\') {
+                if (ALLOW_BACKSLASH) {
+                    b[pos] = (byte) '/';
+                } else {
+                    return false;
+                }
+            }
             if (b[pos] == (byte) 0)
                 return false;
         }
