@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -57,7 +58,7 @@ import org.apache.tomcat.util.compat.JdkCompat;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 371866 $ $Date: 2006-01-24 02:52:54 -0600 (Tue, 24 Jan 2006) $
+ * @version $Revision: 567009 $ $Date: 2007-08-17 04:54:37 -0600 (Fri, 17 Aug 2007) $
  */
 
 public class Response
@@ -129,8 +130,9 @@ public class Response
      */
     public void setConnector(Connector connector) {
         this.connector = connector;
-        if("AJP/1.3".equals(connector.getProtocol())) {
+        if ("AJP/1.3".equals(connector.getProtocol())) {
             // default size to size of one ajp-packet
+            // Reduce HSIZE and Command = 6 Bytes
             outputBuffer = new OutputBuffer(8184);
         } else {
             outputBuffer = new OutputBuffer();
@@ -303,6 +305,12 @@ public class Response
         return outputBuffer.getContentWritten();
     }
 
+    /**
+     * Return the number of bytes actually written to the output stream.
+     */
+    public long getContentCountLong() {
+        return outputBuffer.getContentWrittenLong();
+    }
 
     /**
      * Set the application commit flag.
