@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -38,7 +39,7 @@ import org.apache.commons.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 303666 $ $Date: 2005-01-29 13:38:37 -0600 (Sat, 29 Jan 2005) $
+ * @version $Revision: 478655 $ $Date: 2006-11-23 13:05:58 -0700 (Thu, 23 Nov 2006) $
  */
 
 public class StandardHost
@@ -92,7 +93,13 @@ public class StandardHost
     private String configClass =
         "org.apache.catalina.startup.ContextConfig";
 
+    /**
+     * The Java class name of the default host config configuration class (deployer)
+     */
+    private String hostConfigClass =
+        "org.apache.catalina.startup.HostConfig";
 
+        
     /**
      * The Java class name of the default Context implementation class for
      * deployed web applications.
@@ -120,11 +127,6 @@ public class StandardHost
     private String errorReportValveClass =
         "org.apache.catalina.valves.ErrorReportValve";
 
-    /**
-     * The object name for the errorReportValve.
-     */
-    private ObjectName errorReportValveObjectName = null;
-
     /**
      * The descriptive information string for this implementation.
      */
@@ -132,12 +134,6 @@ public class StandardHost
         "org.apache.catalina.core.StandardHost/1.0";
 
 
-    /**
-     * The live deploy flag for this Host.
-     */
-    private boolean liveDeploy = true;
-
-
     /**
      * Unpack WARs property.
      */
@@ -243,7 +239,31 @@ public class StandardHost
 
     }
 
+    
+    /**
+     * Return the Java class name of the host configuration class (deployer)
+     */
+    public String getHostConfigClass() {
 
+        return (this.hostConfigClass);
+
+    }
+
+
+    /**
+     * Set the Java class name of the host config configuration class (deployer)
+     *
+     * @param hostConfigClass The new host config configuration class (deployer)
+     */
+    public void setHostConfigClass(String hostConfigClass) {
+
+        String oldHostConfigClass = this.hostConfigClass;
+        this.hostConfigClass = hostConfigClass;
+        support.firePropertyChange("hostConfigClass",
+                                   oldHostConfigClass, this.hostConfigClass);
+    }
+
+    
     /**
      * Return the Java class name of the Context implementation class
      * for new web applications.
@@ -690,23 +710,21 @@ public class StandardHost
             && (!errorReportValveClass.equals(""))) {
             try {
                 boolean found = false;
-                if(errorReportValveObjectName != null) {
-                    ObjectName[] names = 
-                        ((StandardPipeline)pipeline).getValveObjectNames();
-                    for (int i=0; !found && i<names.length; i++)
-                        if(errorReportValveObjectName.equals(names[i]))
-                            found = true ;
-                    }
-                    if(!found) {          	
-                        Valve valve = (Valve) Class.forName(errorReportValveClass)
-                        .newInstance();
-                        addValve(valve);
-                        errorReportValveObjectName = ((ValveBase)valve).getObjectName() ;
-                    }
+                Valve[] valves = 
+                    ((StandardPipeline)pipeline).getValves();
+                for (int i=0; !found && i<valves.length; i++)
+                    if(errorReportValveClass.equals(
+                            valves[i].getClass().getName()))
+                        found = true ;
+                if(!found) {          	
+                    Valve valve = (Valve) Class.forName(errorReportValveClass)
+                    .newInstance();
+                    addValve(valve);
+                }
             } catch (Throwable t) {
                 log.error(sm.getString
                     ("standardHost.invalidErrorReportValveClass", 
-                     errorReportValveClass));
+                     errorReportValveClass), t);
             }
         }
         if(log.isInfoEnabled()) {
@@ -758,7 +776,19 @@ public class StandardHost
                 ObjectName serviceName=new ObjectName(domain + 
                                         ":type=Engine");
 
-                HostConfig deployer = new HostConfig();
+                HostConfig deployer = null;
+                try {
+                    String hostConfigClassname = getHostConfigClass();
+                    if (hostConfigClassname != null) {
+                        Class clazz = Class.forName(hostConfigClassname);
+                        deployer = (HostConfig) clazz.newInstance();
+                    } else {
+                        deployer = new HostConfig();
+                    }
+                } catch (Exception e) {
+                    log.warn("Error creating HostConfig for host " + name, e);
+                    throw e;
+                }
                 addLifecycleListener(deployer);                
                 if( mserver.isRegistered( serviceName )) {
                     if(log.isDebugEnabled())
