@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2002,2004-2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -54,7 +55,6 @@ import org.apache.catalina.valves.RemoteHostValve;
 import org.apache.catalina.valves.RequestDumperValve;
 import org.apache.catalina.valves.ValveBase;
 import org.apache.commons.modeler.BaseModelMBean;
-import org.apache.commons.modeler.Registry;
 
 
 /**
@@ -62,7 +62,7 @@ import org.apache.commons.modeler.Registry;
  * <code>org.apache.catalina.core.StandardServer</code> component.</p>
  *
  * @author Amy Roh
- * @version $Revision: 440749 $ $Date: 2006-09-06 11:06:18 -0500 (Wed, 06 Sep 2006) $
+ * @version $Revision: 505759 $ $Date: 2007-02-10 13:20:48 -0700 (Sat, 10 Feb 2007) $
  */
 
 public class MBeanFactory extends BaseModelMBean {
@@ -75,12 +75,6 @@ public class MBeanFactory extends BaseModelMBean {
      */
     private static MBeanServer mserver = MBeanUtils.createServer();
 
-    /**
-     * The configuration information registry for our managed beans.
-     */
-    private static Registry registry = MBeanUtils.createRegistry();
-
-
     // ----------------------------------------------------------- Constructors
 
 
@@ -308,8 +302,8 @@ public class MBeanFactory extends BaseModelMBean {
      * @exception Exception if an MBean cannot be created or registered
      */
     public String createHttpConnector(String parent, String address, int port)
-        throws Exception {
-	return createConnector(parent, address, port, false, false);
+            throws Exception {
+        return createConnector(parent, address, port, false, false);
     }
 
     /**
@@ -325,14 +319,17 @@ public class MBeanFactory extends BaseModelMBean {
      */
     private String createConnector(String parent, String address, int port, boolean isAjp, boolean isSSL)
         throws Exception {
-        Connector retobj = new Connector();
+        Connector retobj;
+        if (isAjp) {
+            retobj = new Connector("AJP/1.3");
+        } else {
+            retobj = new Connector("HTTP/1.1");
+        }
         if ((address!=null) && (address.length()>0)) {
             retobj.setProperty("address", address);
         }
         // Set port number
         retobj.setPort(port);
-        // Set the protocol
-        retobj.setProtocol(isAjp ? "AJP/1.3" : "HTTP/1.1");
         // Set SSL
         retobj.setSecure(isSSL);
         retobj.setScheme(isSSL ? "https" : "http");
@@ -371,7 +368,8 @@ public class MBeanFactory extends BaseModelMBean {
      * @exception Exception if an MBean cannot be created or registered
      */
     public String createJDBCRealm(String parent, String driverName, 
-    	String connectionName, String connectionPassword, String connectionURL)
+            String connectionName, String connectionPassword,
+            String connectionURL)
         throws Exception {
 
         // Create a new JDBCRealm instance
@@ -402,17 +400,21 @@ public class MBeanFactory extends BaseModelMBean {
      * Create a new JNDI Realm.
      *
      * @param parent MBean Name of the associated parent component
+     * @param connectionURL URL to connect to directory
+     * @param connectionName Username to use to establish connection
+     * @param connectionPassword Password to authenticate with
      *
      * @exception Exception if an MBean cannot be created or registered
      */
-    public String createJNDIRealm(String parent,String connectionURL,String connectionName,String connectionPassword)
+    public String createJNDIRealm(String parent, String connectionURL,
+            String connectionName,String connectionPassword)
         throws Exception {
 
          // Create a new JNDIRealm instance
         JNDIRealm realm = new JNDIRealm();
-				realm.setConnectionURL(connectionURL);
-				realm.setConnectionName(connectionName);
-				realm.setConnectionPassword(connectionPassword);
+        realm.setConnectionURL(connectionURL);
+        realm.setConnectionName(connectionName);
+        realm.setConnectionPassword(connectionPassword);
         // Add the new instance to its parent component
         ObjectName pname = new ObjectName(parent);
         ContainerBase containerBase = getParentContainerFromParent(pname);
@@ -429,7 +431,7 @@ public class MBeanFactory extends BaseModelMBean {
 
 
     }
-		
+    
   /**
      * Create a new JAAS Realm.
      *
@@ -442,15 +444,14 @@ public class MBeanFactory extends BaseModelMBean {
 
          // Create a new JAASRealm instance
         JAASRealm realm = new JAASRealm();
-				realm.setAppName(appName);
-				realm.setUserClassNames(userClassNames);
-				realm.setRoleClassNames(roleClassNames);
-				if("true".equals(useContextClassLoader)||"TRUE".equals(useContextClassLoader)){
-						realm.setUseContextClassLoader(true);
-				}
-				else{
-						realm.setUseContextClassLoader(false);
-				}
+        realm.setUserClassNames(userClassNames);
+        realm.setRoleClassNames(roleClassNames);
+        if("true".equals(useContextClassLoader) ||
+                "TRUE".equals(useContextClassLoader)){
+            realm.setUseContextClassLoader(true);
+        } else {
+            realm.setUseContextClassLoader(false);
+        }
         // Add the new instance to its parent component
         ObjectName pname = new ObjectName(parent);
         ContainerBase containerBase = getParentContainerFromParent(pname);
@@ -766,7 +767,7 @@ public class MBeanFactory extends BaseModelMBean {
         host.setUnpackWARs(unpackWARs);
         host.setXmlNamespaceAware(xmlNamespaceAware);
         host.setXmlValidation(xmlValidation);
-	
+  
         // add HostConfig for active reloading
         HostConfig hostConfig = new HostConfig();
         host.addLifecycleListener(hostConfig);
@@ -912,7 +913,6 @@ public class MBeanFactory extends BaseModelMBean {
 
         // Acquire a reference to the component to be removed
         ObjectName oname = new ObjectName(name);
-        Server server = ServerFactory.getServer();
         Service service = getService(oname);
         String port = oname.getKeyProperty("port");
         //String address = oname.getKeyProperty("address");
@@ -1102,7 +1102,6 @@ public class MBeanFactory extends BaseModelMBean {
         // Acquire a reference to the component to be removed
         ObjectName oname = new ObjectName(name);
         ContainerBase container = getParentContainerFromChild(oname);
-        String sequence = oname.getKeyProperty("seq");
         Valve[] valves = (Valve[])container.getValves();
         for (int i = 0; i < valves.length; i++) {
             ObjectName voname = ((ValveBase) valves[i]).getObjectName();
@@ -1114,3 +1113,4 @@ public class MBeanFactory extends BaseModelMBean {
 
 }
 
+
