@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -23,7 +24,6 @@ import java.io.File;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Map;
-import org.apache.catalina.Container;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.util.StringManager;
 import org.apache.commons.logging.Log;
@@ -42,7 +42,7 @@ import org.apache.tomcat.util.digester.Digester;
  * synchronization is performed around accesses to the principals collection.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 373023 $ $Date: 2006-01-27 17:17:43 -0600 (Fri, 27 Jan 2006) $
+ * @version $Revision: 514182 $ $Date: 2007-03-03 17:22:48 +0100 (Sat, 03 Mar 2007) $
  */
 
 public class MemoryRealm  extends RealmBase {
@@ -52,12 +52,6 @@ public class MemoryRealm  extends RealmBase {
     // ----------------------------------------------------- Instance Variables
 
 
-    /**
-     * The Container with which this Realm is associated.
-     */
-    private Container container = null;
-
-
     /**
      * The Digester we will use to process in-memory database files.
      */
@@ -98,12 +92,6 @@ public class MemoryRealm  extends RealmBase {
         StringManager.getManager(Constants.Package);
 
 
-    /**
-     * Has this component been started?
-     */
-    private boolean started = false;
-
-
     // ------------------------------------------------------------- Properties
 
 
@@ -313,7 +301,8 @@ public class MemoryRealm  extends RealmBase {
                 digester.parse(file);
             }
         } catch (Exception e) {
-            throw new LifecycleException("memoryRealm.readXml", e);
+            throw new LifecycleException
+                (sm.getString("memoryRealm.readXml"), e);
         } finally {
             digester.reset();
         }
