@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -57,7 +58,7 @@ import org.apache.commons.modeler.Registry;
  * be subclassed to create more sophisticated Manager implementations.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 395980 $ $Date: 2006-04-21 13:29:21 -0500 (Fri, 21 Apr 2006) $
+ * @version $Revision: 531678 $ $Date: 2007-04-24 04:16:55 +0200 (Tue, 24 Apr 2007) $
  */
 
 public abstract class ManagerBase implements Manager, MBeanRegistration {
@@ -691,16 +692,21 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
             Registry.getRegistry(null, null).unregisterComponent(oname);
         initialized=false;
         oname = null;
+        // Don't clear log since it is required in case attributes are changed
+        // (eg via JMX) whilst the manager is stopped.
     }
     
     public void init() {
         if( initialized ) return;
         initialized=true;        
         
+        // Re-initialise the log to prevent memory leaks on reload in case it
+        // was loaded by the webapp classloader
+        log = LogFactory.getLog(ManagerBase.class);
+        
         if( oname==null ) {
             try {
                 StandardContext ctx=(StandardContext)this.getContainer();
-                Engine eng=(Engine)ctx.getParent().getParent();
                 domain=ctx.getEngineName();
                 distributable = ctx.getDistributable();
                 StandardHost hst=(StandardHost)ctx.getParent();
