@@ -1,15 +1,21 @@
 /*
- * Copyright 1999,2004 The Apache Software Foundation. Licensed under the
- * Apache License, Version 2.0 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of the License
- * at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable
- * law or agreed to in writing, software distributed under the License is
- * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- * KIND, either express or implied. See the License for the specific language
- * governing permissions and limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
-package org.apache.catalina.ssi;
 
+package org.apache.catalina.ssi;
 
 import java.io.BufferedReader;
 import java.io.IOException;
@@ -25,6 +31,7 @@ import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import org.apache.catalina.Globals;
+
 /**
  * Servlet to process SSI requests within a webpage. Mapped to a path from
  * within web.xml.
@@ -33,7 +40,7 @@ import org.apache.catalina.Globals;
  * @author Amy Roh
  * @author Dan Sandberg
  * @author David Becker
- * @version $Revision: 303882 $, $Date: 2005-04-23 05:22:37 -0500 (Sat, 23 Apr 2005) $
+ * @version $Revision: 565195 $, $Date: 2007-08-12 17:00:41 -0600 (Sun, 12 Aug 2007) $
  */
 public class SSIServlet extends HttpServlet {
     /** Debug level for this servlet. */
@@ -61,50 +68,64 @@ public class SSIServlet extends HttpServlet {
         String value = null;
         try {
             value = getServletConfig().getInitParameter("debug");
-            debug = Integer.parseInt(value);
-        } catch (Throwable t) {
-            ;
+            if (value != null) {
+                debug = Integer.parseInt(value);
+            }
+        } catch (Exception e) {
+            log("SSIServlet.init: error reading debug from " + value);
         }
+
         try {
             value = getServletConfig().getInitParameter(
                     "isVirtualWebappRelative");
-            isVirtualWebappRelative = Integer.parseInt(value) > 0?true:false;
-        } catch (Throwable t) {
-            ;
+            if (value != null) {
+                isVirtualWebappRelative = Integer.parseInt(value) > 0?true:false;
+            }
+        } catch (Exception e) {
+            log("SSIServlet.init: error reading isVirtualWebappRelative from"
+                + value);
         }
+
         try {
             value = getServletConfig().getInitParameter("expires");
-            expires = Long.valueOf(value);
+            if (value != null) {
+                expires = Long.valueOf(value);
+            }
         } catch (NumberFormatException e) {
             expires = null;
             log("Invalid format for expires initParam; expected integer (seconds)");
-        } catch (Throwable t) {
-            ;
+        } catch (Exception e) {
+            log("SSIServlet.init: error reading expires from " + value);
         }
+
         try {
             value = getServletConfig().getInitParameter("buffered");
-            buffered = Integer.parseInt(value) > 0?true:false;
-        } catch (Throwable t) {
-            ;
+            if (value != null) {
+                buffered = Integer.parseInt(value) > 0?true:false;
+            }
+        } catch (Exception e) {
+            log("SSIServlet.init: error reading buffered from " + value);
         }
+
         try {
             inputEncoding = getServletConfig().getInitParameter("inputEncoding");
-        } catch (Throwable t) {
-            ;
+        } catch (Exception e) {
+            log("SSIServlet.init: error reading inputEncoding: " + e.getMessage());
         }
+
         try {
             value = getServletConfig().getInitParameter("outputEncoding");
             if (value != null) {
                 outputEncoding = value;
             }
-        } catch (Throwable t) {
-            ;
+        } catch (Exception e) {
+            log("SSIServlet.init: error reading outputEncoding from " + value);
         }
+
         if (debug > 0)
             log("SSIServlet.init() SSI invoker started with 'debug'=" + debug);
     }
 
-
     /**
      * Process and forward the GET request to our <code>requestHandler()</code>*
      * 
