@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2001,2004 The Apache Software Foundation.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -89,6 +90,7 @@ import org.apache.catalina.util.StringManager;
  * It is modeled after the apache syntax:
  * <ul>
  * <li><code>%{xxx}i</code> for incoming headers
+ * <li><code>%{xxx}o</code> for outgoing headers
  * <li><code>%{xxx}c</code> for a specific cookie
  * <li><code>%{xxx}r</code> xxx is an attribute in the ServletRequest
  * <li><code>%{xxx}s</code> xxx is an attribute in the HttpSession
@@ -104,7 +106,8 @@ import org.apache.catalina.util.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Jason Brittain
- * @version $Revision: 303826 $ $Date: 2005-03-31 04:31:54 -0600 (Thu, 31 Mar 2005) $
+ * @author Peter Rossbach
+ * @version $Revision: 567009 $ $Date: 2007-08-17 04:54:37 -0600 (Fri, 17 Aug 2007) $
  */
 
 public class AccessLogValve
@@ -119,7 +122,7 @@ public class AccessLogValve
      * Construct a new instance of this class with default property values.
      */
     public AccessLogValve() {
-
+        
         super();
         setPattern("common");
 
@@ -598,8 +601,7 @@ public class AccessLogValve
 
             result.append(space);
 
-            int length = response.getContentCount();
-
+            long length = response.getContentCountLong() ;
             if (length <= 0)
                 value = "-";
             else
@@ -804,13 +806,13 @@ public class AccessLogValve
                 value = "127.0.0.1";
             }
         } else if (pattern == 'b') {
-            int length = response.getContentCount();
+            long length = response.getContentCountLong() ;
             if (length <= 0)
                 value = "-";
             else
                 value = "" + length;
         } else if (pattern == 'B') {
-            value = "" + response.getContentLength();
+            value = "" + response.getContentCountLong();
         } else if (pattern == 'h') {
             value = request.getRemoteHost();
         } else if (pattern == 'H') {
@@ -922,11 +924,22 @@ public class AccessLogValve
                 else
                     value= "??";
                 break;
-/*
-            // Someone please make me work
             case 'o':
+                if (null != response) {
+                    String[] values = response.getHeaderValues(header);
+                    if(values.length > 0) {
+                        StringBuffer buf = new StringBuffer() ;
+                        for (int i = 0; i < values.length; i++) {
+                            String string = values[i];
+                            buf.append(string) ;
+                            if(i+1<values.length)
+                                buf.append(",");
+                        }
+                        value = buf.toString();
+                    }
+                } else
+                    value= "??";
                 break;
-*/
             case 'c':
                  Cookie[] c = request.getCookies();
                  for (int i=0; c != null && i < c.length; i++){
