@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2001,2004 The Apache Software Foundation.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -129,7 +130,7 @@ import org.apache.commons.logging.LogFactory;
  *
  *
  * @author Tim Funk
- * @version $Revision: 303601 $ $Date: 2004-12-23 17:58:07 -0600 (Thu, 23 Dec 2004) $
+ * @version $Revision: 567009 $ $Date: 2007-08-17 04:54:37 -0600 (Fri, 17 Aug 2007) $
  */
 
 public class ExtendedAccessLogValve
@@ -583,7 +584,7 @@ public class ExtendedAccessLogValve
                     else if (FieldInfo.SPECIAL_TIME==fieldInfos[i].location)
                         result.append(timeFormatter.format(date));
                     else if (FieldInfo.SPECIAL_BYTES==fieldInfos[i].location) {
-                        int length = response.getContentCount();
+                        long length = response.getContentCountLong() ;
                         if (length > 0)
                             result.append(length);
                         else
@@ -743,7 +744,7 @@ public class ExtendedAccessLogValve
                             FieldInfo.X_LOC_REQUESTEDSESSIONIDVALID) {
                     return wrap(""+request.isRequestedSessionIdValid());
                 } else if (fieldInfo.location==FieldInfo.X_LOC_CONTENTLENGTH) {
-                    return wrap(""+request.getContentLength());
+                    return wrap(""+request.getCoyoteRequest().getContentLengthLong());
                 } else if (fieldInfo.location==
                             FieldInfo.X_LOC_CHARACTERENCODING) {
                     return wrap(request.getCharacterEncoding());
