@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -74,7 +75,6 @@ import org.apache.catalina.util.StringManager;
  * <pre>
  * CREATE TABLE access (
  * id INT UNSIGNED AUTO_INCREMENT NOT NULL,
- * ts TIMESTAMP NOT NULL,
  * remoteHost CHAR(15) NOT NULL,
  * userName CHAR(15),
  * timestamp TIMESTAMP NOT NULL,
@@ -86,13 +86,17 @@ import org.apache.catalina.util.StringManager;
  * referer VARCHAR(128),
  * userAgent VARCHAR(128),
  * PRIMARY KEY (id),
- * INDEX (ts),
+ * INDEX (timestamp),
  * INDEX (remoteHost),
  * INDEX (virtualHost),
  * INDEX (query),
  * INDEX (userAgent)
  * );
  * </pre>
+ * <p>Set JDBCAccessLogValve attribute useLongContentLength="true" as you have more then 4GB outputs. 
+ * Please, use long SQL datatype at access.bytes attribute.
+ * The datatype of bytes at oracle is <i>number</i> and other databases use <i>bytes BIGINT NOT NULL</i>.</p>
+ * 
  * <p>
  * If the table is created as above, its name and the field names don't need 
  * to be defined.
@@ -115,7 +119,6 @@ public final class JDBCAccessLogValve
 
     // ----------------------------------------------------------- Constructors
 
-
     /**
      * Class constructor. Initializes the fields with the default values.
      * The defaults are:
@@ -162,7 +165,12 @@ public final class JDBCAccessLogValve
 
     // ----------------------------------------------------- Instance Variables
 
-
+   /**
+    * Use long contentLength as you have more 4 GB output.
+    * @since 5.5.25
+    */
+    protected boolean useLongContentLength = false ;
+    
    /**
      * The connection username to use when trying to connect to the database.
      */
@@ -419,6 +427,12 @@ public final class JDBCAccessLogValve
         this.resolveHosts = new Boolean(resolveHosts).booleanValue();
     }
 
+    /**
+     * @param useLongContentLength the useLongContentLength to set
+     */
+    public void setUseLongContentLength(boolean useLongContentLength) {
+        this.useLongContentLength = useLongContentLength;
+    }
 
     // --------------------------------------------------------- Public Methods
 
@@ -449,24 +463,12 @@ public final class JDBCAccessLogValve
         String query="";
         if(request != null)
             query = request.getRequestURI();
-        int bytes = response.getContentCount();
+        
+        long bytes = response.getContentCountLong() ;
         if(bytes < 0)
             bytes = 0;
         int status = response.getStatus();
-        if (pattern.equals("combined")) {
-                String virtualHost = "";
-                if(request != null)
-                    virtualHost = request.getServerName();
-                String method = "";
-                if(request != null)
-                    method = request.getMethod();
-                String referer = "";
-                if(request != null)
-                    referer = request.getHeader("referer");
-                String userAgent = "";
-                if(request != null)
-                    userAgent = request.getHeader("user-agent");
-        }
+
         synchronized (this) {
           int numberOfTries = 2;
           while (numberOfTries>0) {
@@ -478,7 +480,14 @@ public final class JDBCAccessLogValve
                 ps.setTimestamp(3, new Timestamp(getCurrentTimeMillis()));
                 ps.setString(4, query);
                 ps.setInt(5, status);
-                ps.setInt(6, bytes);
+                
+                if(useLongContentLength) {
+                    ps.setLong(6, bytes);                
+                } else {
+                    if (bytes > Integer.MAX_VALUE)
+                        bytes = -1 ;
+                    ps.setInt(6, (int) bytes);
+                }               
                 if (pattern.equals("combined")) {
      
                       String virtualHost = "";
