@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004-2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -67,7 +68,7 @@ import org.apache.catalina.core.StandardContext;
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
  * @author Peter Rossbach
- * @version $Revision: 408460 $ $Date: 2006-05-21 13:26:01 -0500 (Sun, 21 May 2006) $
+ * @version $Revision: 565650 $ $Date: 2007-08-14 02:04:19 -0600 (Tue, 14 Aug 2007) $
  */
 
 public class DeltaManager extends ManagerBase implements Lifecycle,
@@ -143,6 +144,17 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
      */
     private int sendAllSessionsWaitTime = 2 * 1000 ; 
 
+    /**
+     * Send session access message every updateActiveInterval sec 
+     */
+    private int updateActiveInterval = 60;
+
+    /**
+     * The default session maxInactiveInterval + tolerance time interval, in seconds, between client requests before
+     * the servlet container may invalidate a backup session from other cluster node.
+     */
+    private int expireTolerance = 300;
+
     private ArrayList receivedMessageQueue = new ArrayList() ;
     
     private boolean receiverQueue = false ;
@@ -236,6 +248,22 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
 
     }
 
+    public int getUpdateActiveInterval() {
+        return updateActiveInterval;
+    }
+
+    public void setUpdateActiveInterval(int updateActiveInterval) {
+        this.updateActiveInterval = updateActiveInterval;
+    }
+
+    public int getExpireTolerance() {
+        return expireTolerance;
+    }
+
+    public void setExpireTolerance(int expireTolerance) {
+        this.expireTolerance = expireTolerance;
+    }
+
     /**
      * @return Returns the counterSend_EVT_GET_ALL_SESSIONS.
      */
@@ -619,6 +647,7 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
         }
 
         DeltaSession session = (DeltaSession) super.createSession(sessionId) ;
+        session.setExpireTolerance(this.expireTolerance);
         if (distribute) {
             sendCreateSession(session.getId(), session);
         }
@@ -636,7 +665,7 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
      * @param session
      */
     protected void sendCreateSession(String sessionId, DeltaSession session) {
-        if(cluster.getMembers().length > 0 ) {
+        if(cluster != null && cluster.getMembers().length > 0 ) {
             SessionMessage msg = new SessionMessageImpl(getName(),
                     SessionMessage.EVT_SESSION_CREATED, null, sessionId,
                     sessionId + "-" + System.currentTimeMillis());
@@ -746,14 +775,6 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
                 session.setManager(this);
                 session.setValid(true);
                 session.setPrimarySession(false);
-                //in case the nodes in the cluster are out of
-                //time synch, this will make sure that we have the
-                //correct timestamp, isValid returns true, cause
-                // accessCount=1
-                session.access();
-                //make sure that the session gets ready to expire if
-                // needed
-                session.setAccessCount(0);
                 session.resetDeltaRequest();
                 // FIXME How inform other session id cache like SingleSignOn
                 // increment sessionCounter to correct stats report
@@ -932,6 +953,8 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
         if (started) {
             return;
         }
+        if(log.isInfoEnabled())
+            log.info("Starting clustering manager...:"+getName());
         started = true;
         lifecycle.fireLifecycleEvent(START_EVENT, null);
 
@@ -1325,7 +1348,9 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
             if ((msg == null)) {
                 long replDelta = System.currentTimeMillis()
                         - session.getLastTimeReplicated();
-                if (replDelta > (getMaxInactiveInterval() * 1000)) {
+                if (replDelta >= updateActiveInterval*1000 ||
+                    (getMaxInactiveInterval()>=0 &&
+                    replDelta >= getMaxInactiveInterval()*1000)) {
                     counterSend_EVT_SESSION_ACCESSED++;
                     msg = new SessionMessageImpl(getName(),
                             SessionMessage.EVT_SESSION_ACCESSED, null,
@@ -1398,14 +1423,17 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
      *            session id
      */
     protected void sessionExpired(String id) {
-        counterSend_EVT_SESSION_EXPIRED++ ;
-        SessionMessage msg = new SessionMessageImpl(getName(),
-                SessionMessage.EVT_SESSION_EXPIRED, null, id, id
-                        + "-EXPIRED-MSG");
-        if (log.isDebugEnabled())
-            log.debug(sm.getString("deltaManager.createMessage.expire",
-                    getName(), id));
-        send(msg);
+        // FIX BUG 42720 : don't send a message if no cluster member exists.
+        if (cluster != null && cluster.getMembers().length > 0) {
+            counterSend_EVT_SESSION_EXPIRED++ ;
+            SessionMessage msg = new SessionMessageImpl(getName(),
+                    SessionMessage.EVT_SESSION_EXPIRED, null, id, id
+                    + "-EXPIRED-MSG");
+            if (log.isDebugEnabled())
+                log.debug(sm.getString("deltaManager.createMessage.expire",
+                        getName(), id));
+            send(msg);
+        }
     }
 
     /**
@@ -1492,7 +1520,7 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
             if (log.isDebugEnabled())
                 log.debug(sm.getString("deltaManager.receiveMessage.eventType",
                         getName(), msg.getEventTypeString(), sender));
- 
+
             switch (msg.getEventType()) {
             case SessionMessage.EVT_GET_ALL_SESSIONS: {
                 handleGET_ALL_SESSIONS(msg,sender);
@@ -1631,6 +1659,9 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
         session.setValid(true);
         session.setPrimarySession(false);
         session.setCreationTime(msg.getTimestamp());
+        // use container maxInactiveInterval so that session will expire correctly in case of primary transfer
+        session.setMaxInactiveInterval(getMaxInactiveInterval());
+        session.setExpireTolerance(this.expireTolerance);
         session.access();
         if(notifySessionListenersOnReplication)
             session.setId(msg.getSessionID());
