@@ -1,17 +1,18 @@
 /*
- * Copyright 1999,2005 The Apache Software Foundation.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
- * Licensed under the Apache License, Version 2.0 (the "License"); you may not
- * use this file except in compliance with the License. You may obtain a copy of
- * the License at
- * 
- * http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  * 
  * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
- * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
- * License for the specific language governing permissions and limitations under
- * the License.
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.catalina.cluster.tcp;
@@ -19,7 +20,9 @@ package org.apache.catalina.cluster.tcp;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.net.InetAddress;
+import java.net.InetSocketAddress;
 import java.net.Socket;
+import java.net.SocketAddress;
 import java.net.SocketException;
 
 import org.apache.catalina.cluster.ClusterMessage;
@@ -32,7 +35,7 @@ import org.apache.catalina.util.StringManager;
  * 
  * @author Peter Rossbach
  * @author Filip Hanik
- * @version $Revision: 418134 $ $Date: 2006-06-29 15:54:33 -0500 (Thu, 29 Jun 2006) $
+ * @version $Revision: 567298 $ $Date: 2007-08-18 11:14:07 -0600 (Sat, 18 Aug 2007) $
  * @since 5.5.7
  */
 public class DataSender implements IDataSender {
@@ -51,7 +54,7 @@ public class DataSender implements IDataSender {
     /**
      * The descriptive information about this implementation.
      */
-    private static final String info = "DataSender/2.1";
+    private static final String info = "DataSender/2.2";
 
     /**
      * receiver address
@@ -89,11 +92,17 @@ public class DataSender implements IDataSender {
      */
     private SenderState senderState = new SenderState();
 
-    /**
-     * wait time for ack
-     */
-    private long ackTimeout;
-
+    private int rxBufSize = 25188;
+    private int txBufSize = 43800;
+    private int timeout = 30000;
+    private boolean tcpNoDelay = true;
+    private boolean soKeepAlive = false;
+    private boolean ooBInline = true;
+    private boolean soReuseAddress = true;
+    private boolean soLingerOn = false;
+    private int soLingerTime = 3;
+    private int soTrafficClass = 0x04 | 0x08 | 0x010;
+     
     /**
      * number of requests
      */
@@ -212,7 +221,7 @@ public class DataSender implements IDataSender {
     /**
      * After failure make a resend
      */
-    private boolean resend = false ;
+    private boolean resend = true ;
     
     // ------------------------------------------------------------- Constructor
     
@@ -486,12 +495,178 @@ public class DataSender implements IDataSender {
             this.senderState.setReady();
     }
 
-    public long getAckTimeout() {
-        return ackTimeout;
+    /**
+     * @return the ooBInline
+     * @since 5.5.25
+     */
+    public boolean isOoBInline() {
+        return ooBInline;
     }
 
+    /**
+     * @param ooBInline the ooBInline to set
+     * @since 5.5.25
+     */
+    public void setOoBInline(boolean ooBInline) {
+        this.ooBInline = ooBInline;
+    }
+
+    /**
+     * @return the rxBufSize
+     * @since 5.5.25
+     */
+    public int getRxBufSize() {
+        return rxBufSize;
+    }
+
+    /**
+     * @param rxBufSize the rxBufSize to set
+     * @since 5.5.25
+     */
+    public void setRxBufSize(int rxBufSize) {
+        this.rxBufSize = rxBufSize;
+    }
+
+    /**
+     * @return the soKeepAlive
+     * @since 5.5.25
+     */
+    public boolean isSoKeepAlive() {
+        return soKeepAlive;
+    }
+
+    /**
+     * @param soKeepAlive the soKeepAlive to set
+     * @since 5.5.25
+     */
+    public void setSoKeepAlive(boolean soKeepAlive) {
+        this.soKeepAlive = soKeepAlive;
+    }
+
+    /**
+     * @return the soLingerOn
+     * @since 5.5.25
+     */
+    public boolean isSoLingerOn() {
+        return soLingerOn;
+    }
+
+    /**
+     * @param soLingerOn the soLingerOn to set
+     * @since 5.5.25
+     */
+    public void setSoLingerOn(boolean soLingerOn) {
+        this.soLingerOn = soLingerOn;
+    }
+
+    /**
+     * @return the soLingerTime
+     * @since 5.5.25
+     */
+    public int getSoLingerTime() {
+        return soLingerTime;
+    }
+
+    /**
+     * @param soLingerTime the soLingerTime to set
+     * @since 5.5.25
+     */
+    public void setSoLingerTime(int soLingerTime) {
+        this.soLingerTime = soLingerTime;
+    }
+
+    /**
+     * @return the soReuseAddress
+     * @since 5.5.25
+     */
+    public boolean isSoReuseAddress() {
+        return soReuseAddress;
+    }
+
+    /**
+     * @param soReuseAddress the soReuseAddress to set
+     * @since 5.5.25
+     */
+    public void setSoReuseAddress(boolean soReuseAddress) {
+        this.soReuseAddress = soReuseAddress;
+    }
+
+    /**
+     * @return the soTrafficClass
+     * @since 5.5.25
+     */
+    public int getSoTrafficClass() {
+        return soTrafficClass;
+    }
+
+    /**
+     * @param soTrafficClass the soTrafficClass to set
+     * @since 5.5.25
+     */
+    public void setSoTrafficClass(int soTrafficClass) {
+        this.soTrafficClass = soTrafficClass;
+    }
+
+    /**
+     * @return the tcpNoDelay
+     * @since 5.5.25
+     */
+    public boolean isTcpNoDelay() {
+        return tcpNoDelay;
+    }
+
+    /**
+     * @param tcpNoDelay the tcpNoDelay to set
+     * @since 5.5.25
+     */
+    public void setTcpNoDelay(boolean tcpNoDelay) {
+        this.tcpNoDelay = tcpNoDelay;
+    }
+
+    /**
+     * @return the timeout
+     * @since 5.5.25
+     */
+    public int getTimeout() {
+        return timeout;
+    }
+
+    /**
+     * @param timeout the timeout to set
+     * @since 5.5.25
+     */
+    public void setTimeout(int timeout) {
+        this.timeout = timeout;
+    }
+
+    /**
+     * @return the txBufSize
+     * @since 5.5.25
+     */
+    public int getTxBufSize() {
+        return txBufSize;
+    }
+
+    /**
+     * @param txBufSize the txBufSize to set
+     * @since 5.5.25
+     */
+    public void setTxBufSize(int txBufSize) {
+        this.txBufSize = txBufSize;
+    }
+
+    /**
+     * @deprecated since 5.5.25 use timeout instead
+     */
+    public long getAckTimeout() {
+        return timeout;
+    }
+
+    /**
+     * @deprecated since 5.5.25 use timeout instead
+     */
     public void setAckTimeout(long ackTimeout) {
-        this.ackTimeout = ackTimeout;
+        this.timeout = (int)ackTimeout;
     }
 
     public long getKeepAliveTimeout() {
@@ -690,7 +865,7 @@ public class DataSender implements IDataSender {
        try {
             createSocket();
             if (isWaitForAck())
-                socket.setSoTimeout((int) ackTimeout);
+                socket.setSoTimeout((int) getTimeout());
             isSocketConnected = true;
             socketOpenCounter++;
             this.keepAliveCount = 0;
@@ -709,11 +884,30 @@ public class DataSender implements IDataSender {
      }
 
     /**
+     * Create new Socket and use actTimeout (>0) as connect timeout
+     * 
      * @throws IOException
      * @throws SocketException
      */
     protected void createSocket() throws IOException, SocketException {
-        socket = new Socket(getAddress(), getPort());
+        SocketAddress sockAddr = new InetSocketAddress(getAddress(), getPort());
+        socket = new Socket();
+        int timeout = getTimeout();
+        if (timeout > 0) {
+           socket.connect(sockAddr, timeout);
+           socket.setSoTimeout(timeout);
+        } else {
+           socket.connect(sockAddr);
+        }
+        socket.setSendBufferSize(getTxBufSize());
+        socket.setReceiveBufferSize(getRxBufSize());
+        socket.setTcpNoDelay(isTcpNoDelay());
+        socket.setKeepAlive(isSoKeepAlive());
+        socket.setReuseAddress(isSoReuseAddress());
+        socket.setOOBInline(isOoBInline());
+        socket.setSoLinger(isSoLingerOn(),getSoLingerTime());
+        socket.setTrafficClass(getSoTrafficClass());
+
     }
 
     /**
@@ -882,7 +1076,7 @@ public class DataSender implements IDataSender {
             out.write(XByteBuffer.createDataPackage(message,data.getCompress()));
             out.flush();
             if (isWaitForAck())
-                waitForAck(ackTimeout);
+                waitForAck(getTimeout());
         } finally {
             synchronized(this) {
                 isMessageTransferStarted = false ;
@@ -928,13 +1122,11 @@ public class DataSender implements IDataSender {
             missingAckCounter++;
             String errmsg = sm.getString("IDataSender.ack.missing", getAddress(),
                                          new Integer(socket.getLocalPort()), 
-                                         new Long(this.ackTimeout));
+                                         new Long(this.timeout));
             if ( !this.isSuspect() ) {
                 this.setSuspect(true);
-                if ( log.isWarnEnabled() ) log.warn(errmsg, x);
-            } else {
-                if ( log.isDebugEnabled() )log.debug(errmsg, x);
             }
+            if ( log.isDebugEnabled() ) log.debug(errmsg, x);
             throw x;
         } finally {
             if(doWaitAckStats) {
