@@ -1,9 +1,10 @@
 /*
- * Copyright 1999,2004-2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -66,7 +67,7 @@ import org.apache.tomcat.util.IntrospectionUtils;
  * @author Filip Hanik
  * @author Remy Maucherat
  * @author Peter Rossbach
- * @version $Revision: 379550 $, $Date: 2006-02-21 12:06:35 -0600 (Tue, 21 Feb 2006) $
+ * @version $Revision: 522314 $, $Date: 2007-03-25 11:46:37 -0600 (Sun, 25 Mar 2007) $
  */
 public class SimpleTcpCluster 
     implements CatalinaCluster, Lifecycle, LifecycleListener, IDynamicProperty,
@@ -545,18 +546,25 @@ public class SimpleTcpCluster
     }
 
     /**
-     * @param name
-     * @param manager
-     * @return
+     * Returns the name for the cluster manager.  This name
+     * depends on whether the cluster manager is defined inside
+     * a Host or an Engine.
+     * Should the cluster be defined in the Engine element, the manager name
+     * is Host.getName+"#"+Context.getManager().getName()
+     * otherwise the name is simply Context.getManager().getName()
+     * @param name The current name
+     * @param manager The cluster manager implementation
+     * @return The name to use for the cluster manager
      */
     public String getManagerName(String name, Manager manager) {
         String clusterName = name ;
+        if ( clusterName == null ) clusterName = manager.getContainer().getName();
         if(getContainer() instanceof Engine) {
             Container context = manager.getContainer() ;
             if(context != null && context instanceof Context) {
                 Container host = ((Context)context).getParent();
-                if(host != null && host instanceof Host)
-                    clusterName = host.getName()  + name ;
+                if(host != null && host instanceof Host && clusterName!=null && !(clusterName.indexOf("#")>=0))
+                    clusterName = host.getName() +"#" + clusterName ;
             }
         }
         return clusterName;
