@@ -1,9 +1,10 @@
 /*
- * Copyright 2001-2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -17,27 +18,21 @@
 package org.apache.webapp.admin.connector;
 
 import java.net.URLEncoder;
-import java.util.Iterator;
 import java.util.Locale;
 import java.io.IOException;
 import javax.management.Attribute;
 import javax.management.MBeanServer;
-import javax.management.MBeanServerFactory;
-import javax.management.QueryExp;
-import javax.management.Query;
-import javax.management.ObjectInstance;
 import javax.management.ObjectName;
-import javax.management.JMException;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 import org.apache.struts.action.Action;
-import org.apache.struts.action.ActionError;
-import org.apache.struts.action.ActionErrors;
 import org.apache.struts.action.ActionForm;
 import org.apache.struts.action.ActionForward;
 import org.apache.struts.action.ActionMapping;
+import org.apache.struts.action.ActionMessage;
+import org.apache.struts.action.ActionMessages;
 import org.apache.struts.util.MessageResources;
 import org.apache.webapp.admin.ApplicationServlet;
 import org.apache.webapp.admin.TomcatTreeBuilder;
@@ -50,7 +45,7 @@ import org.apache.webapp.admin.TreeControlNode;
  * <em>Edit Connector</em> transactions.
  *
  * @author Manveen Kaur
- * @version $Revision: 303390 $ $Date: 2004-10-18 01:37:56 -0500 (Mon, 18 Oct 2004) $
+ * @version $Revision: 528826 $ $Date: 2007-04-14 18:17:12 +0200 (Sat, 14 Apr 2007) $
  */
 
 public final class SaveConnectorAction extends Action {
@@ -148,9 +143,9 @@ public final class SaveConnectorAction extends Action {
                 // Ensure that the requested connector name and port is unique
                 if (mBServer.isRegistered(oname) ||
                     (!mBServer.queryNames(search, null).isEmpty())) {
-                    ActionErrors errors = new ActionErrors();
+                    ActionMessages errors = new ActionMessages();
                     errors.add("connectorName",
-                               new ActionError("error.connectorName.exists"));
+                               new ActionMessage("error.connectorName.exists"));
                     saveErrors(request, errors);
                     return (new ActionForward(mapping.getInput()));
                 }
@@ -297,24 +292,6 @@ public final class SaveConnectorAction extends Action {
             }
             mBServer.setAttribute(coname,
                                   new Attribute("redirectPort", new Integer(redirectPort))); 
-            attribute = "minProcessors";
-            int minProcessors = 5;
-            try {
-                minProcessors = Integer.parseInt(cform.getMinProcessorsText());
-            } catch (Throwable t) {
-                minProcessors = 5;
-            }
-            //mBServer.setAttribute(coname,
-            //                      new Attribute("minProcessors", new Integer(minProcessors))); 
-            attribute = "maxProcessors";
-            int maxProcessors = 20;
-            try {
-                maxProcessors = Integer.parseInt(cform.getMaxProcessorsText());
-            } catch (Throwable t) {
-                maxProcessors = 20;
-            }
-            //mBServer.setAttribute(coname,
-            //                      new Attribute("maxProcessors", new Integer(maxProcessors))); 
        
             attribute = "maxKeepAliveRequests";
             int maxKeepAliveRequests = 100;
@@ -413,43 +390,51 @@ public final class SaveConnectorAction extends Action {
             
             // HTTPS specific properties
             if("HTTPS".equalsIgnoreCase(connectorType)) {
-                attribute = "algorithm";
                 String algorithm = cform.getAlgorithm();
                 if ((algorithm != null) && (algorithm.length()>0)) 
                     mBServer.setAttribute(coname,
                               new Attribute("algorithm", algorithm));  
                 
-                attribute = "clientAuth";              
                 mBServer.setAttribute(coname,
                               new Attribute("clientAuth", 
                                              cform.getClientAuthentication()));   
                 
-                attribute = "ciphers";
                 String ciphers = cform.getCiphers();
                 if ((ciphers != null) && (ciphers.length()>0)) 
                     mBServer.setAttribute(coname,
                               new Attribute("ciphers", ciphers));           
                 
-                attribute = "keystoreFile";
                 String keyFile = cform.getKeyStoreFileName();
                 if ((keyFile != null) && (keyFile.length()>0)) 
                     mBServer.setAttribute(coname,
                               new Attribute("keystoreFile", keyFile));            
                 
-                attribute = "keystorePass";
                 String keyPass = cform.getKeyStorePassword();
                 if ((keyPass != null) && (keyPass.length()>0)) 
                     mBServer.setAttribute(coname,
                               new Attribute("keystorePass", keyPass));                 
                 // request.setAttribute("warning", "connector.keyPass.warning");  
                 
-                attribute = "keystoreType";
                 String keyType = cform.getKeyStoreType();
                 if ((keyType != null) && (keyType.length()>0)) 
                     mBServer.setAttribute(coname,
                               new Attribute("keystoreType", keyType));   
                 
-                attribute = "sslProtocol";
+                String trustFile = cform.getTrustStoreFileName();
+                if ((trustFile != null) && (trustFile.length()>0)) 
+                    mBServer.setAttribute(coname,
+                              new Attribute("truststoreFile", trustFile));            
+                
+                String trustPass = cform.getTrustStorePassword();
+                if ((trustPass != null) && (trustPass.length()>0)) 
+                    mBServer.setAttribute(coname,
+                              new Attribute("truststorePass", trustPass));                 
+                
+                String trustType = cform.getTrustStoreType();
+                if ((trustType != null) && (trustType.length()>0)) 
+                    mBServer.setAttribute(coname,
+                              new Attribute("truststoreType", trustType));   
+                
                 String sslProtocol = cform.getSslProtocol();
                 if ((sslProtocol != null) && (sslProtocol.length()>0)) 
                     mBServer.setAttribute(coname,
