@@ -1,9 +1,10 @@
 /*
- * Copyright 2001-2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -29,11 +30,11 @@ import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 import org.apache.commons.modeler.Registry;
 import org.apache.struts.action.Action;
-import org.apache.struts.action.ActionError;
-import org.apache.struts.action.ActionErrors;
 import org.apache.struts.action.ActionForm;
 import org.apache.struts.action.ActionForward;
 import org.apache.struts.action.ActionMapping;
+import org.apache.struts.action.ActionMessage;
+import org.apache.struts.action.ActionMessages;
 import org.apache.struts.util.MessageResources;
 import org.apache.webapp.admin.ApplicationServlet;
 import org.apache.webapp.admin.TomcatTreeBuilder;
@@ -41,13 +42,12 @@ import org.apache.webapp.admin.TreeControl;
 import org.apache.webapp.admin.TreeControlNode;
 
 
-
 /**
  * The <code>Action</code> that completes <em>Add Context</em> and
  * <em>Edit Context</em> transactions.
  *
  * @author Manveen Kaur
- * @version $Revision: 303641 $ $Date: 2005-01-14 17:55:41 -0600 (Fri, 14 Jan 2005) $
+ * @version $Revision: 479033 $ $Date: 2006-11-24 16:00:06 -0700 (Fri, 24 Nov 2006) $
  */
 
 public final class SaveContextAction extends Action {
@@ -159,9 +159,9 @@ public final class SaveContextAction extends Action {
                                     ",J2EEApplication=none,J2EEServer=none");                   
                 
                 if (mBServer.isRegistered(oname)) {
-                    ActionErrors errors = new ActionErrors();
+                    ActionMessages errors = new ActionMessages();
                     errors.add("contextName",
-                               new ActionError("error.contextName.exists"));
+                               new ActionMessage("error.contextName.exists"));
                     saveErrors(request, errors);
                     return (new ActionForward(mapping.getInput()));
                 }
@@ -207,7 +207,8 @@ public final class SaveContextAction extends Action {
                     values[0] = cObjectName;
                     mBServer.invoke(fname, operation, values, 
                         removeContextTypes);
-                    Registry.getRegistry().unregisterComponent(new ObjectName(cObjectName));
+                    Registry.getRegistry(null, null).unregisterComponent(
+                            new ObjectName(cObjectName));
                     request.setAttribute("warning", "error.context.directory");
                     return (mapping.findForward("Save Unsuccessful"));
                 }
