@@ -1,9 +1,10 @@
 /*
- * Copyright 2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -17,21 +18,17 @@
 package org.apache.webapp.admin.realm;
 
 import javax.servlet.http.HttpServletRequest;
-import org.apache.struts.action.ActionError;
 import org.apache.struts.action.ActionErrors;
-import org.apache.struts.action.ActionForm;
 import org.apache.struts.action.ActionMapping;
-import java.net.InetAddress;
-import java.util.List;
+import org.apache.struts.action.ActionMessage;
 
-import org.apache.webapp.admin.ApplicationServlet;
-import org.apache.webapp.admin.LabelValueBean;
+import java.util.List;
 
 /**
  * Form bean for the datasource realm page.
  *
  * @author Amy Roh
- * @version $Revision: 302986 $ $Date: 2004-06-27 21:14:52 -0500 (Sun, 27 Jun 2004) $
+ * @version $Revision: 479037 $ $Date: 2006-11-24 16:01:05 -0700 (Fri, 24 Nov 2006) $
  */
 
 public final class DataSourceRealmForm extends RealmForm {
@@ -319,44 +316,36 @@ public final class DataSourceRealmForm extends RealmForm {
         
         ActionErrors errors = new ActionErrors();
         
-        String submit = request.getParameter("submit");
-        //String type = request.getParameter("realmType");
-        
         // front end validation when save is clicked.        
-         //if (submit != null) {
-             // the following fields are required.
-            
-            if ((dataSourceName == null) || (dataSourceName.length() < 1)) {
-                errors.add("dataSourceName",
-                new ActionError("error.dataSourceName.required"));
-            }
-         
-            if ((roleNameCol == null) || (roleNameCol.length() < 1)) {
-                errors.add("roleNameCol",
-                new ActionError("error.roleNameCol.required"));
-            }
+        if ((dataSourceName == null) || (dataSourceName.length() < 1)) {
+            errors.add("dataSourceName",
+            new ActionMessage("error.dataSourceName.required"));
+        }
+     
+        if ((roleNameCol == null) || (roleNameCol.length() < 1)) {
+            errors.add("roleNameCol",
+            new ActionMessage("error.roleNameCol.required"));
+        }
 
-            if ((userCredCol == null) || (userCredCol.length() < 1)) {
-                errors.add("userCredCol",
-                new ActionError("error.userCredCol.required"));
-            }
+        if ((userCredCol == null) || (userCredCol.length() < 1)) {
+            errors.add("userCredCol",
+            new ActionMessage("error.userCredCol.required"));
+        }
+    
+        if ((userNameCol == null) || (userNameCol.length() < 1)) {
+            errors.add("userNameCol",
+            new ActionMessage("error.userNameCol.required"));
+        }
         
-            if ((userNameCol == null) || (userNameCol.length() < 1)) {
-                errors.add("userNameCol",
-                new ActionError("error.userNameCol.required"));
-            }
-            
-            if ((userRoleTable == null) || (userRoleTable.length() < 1)) {
-                errors.add("userRoleTable",
-                new ActionError("error.userRoleTable.required"));
-            }
-        
-            if ((userTable == null) || (userTable.length() < 1)) {
-                errors.add("userTable",
-                new ActionError("error.userTable.required"));
-            }
-            
-        //}
+        if ((userRoleTable == null) || (userRoleTable.length() < 1)) {
+            errors.add("userRoleTable",
+            new ActionMessage("error.userRoleTable.required"));
+        }
+    
+        if ((userTable == null) || (userTable.length() < 1)) {
+            errors.add("userTable",
+            new ActionMessage("error.userTable.required"));
+        }
                  
         return errors;
     }
