@@ -1,9 +1,10 @@
 /*
- * Copyright 2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -18,20 +19,16 @@ package org.apache.webapp.admin.server;
 
 
 import javax.servlet.http.HttpServletRequest;
-import org.apache.struts.action.ActionError;
 import org.apache.struts.action.ActionErrors;
 import org.apache.struts.action.ActionForm;
 import org.apache.struts.action.ActionMapping;
-import org.apache.webapp.admin.ApplicationServlet;
-import org.apache.webapp.admin.TomcatTreeBuilder;
-
-import java.util.List;
+import org.apache.struts.action.ActionMessage;
 
 /**
  * Form bean for the server form page.  
  * @author Patrick Luby
  * @author Manveen Kaur
- * @version $Revision: 302986 $ $Date: 2004-06-27 21:14:52 -0500 (Sun, 27 Jun 2004) $
+ * @version $Revision: 479042 $ $Date: 2006-11-24 16:02:41 -0700 (Fri, 24 Nov 2006) $
  */
 
 public final class ServerForm extends ActionForm {
@@ -163,35 +160,30 @@ public final class ServerForm extends ActionForm {
         
         ActionErrors errors = new ActionErrors();
         
-        String submit = request.getParameter("submit");
-        //if (submit != null) {
-            
-            // check for portNumber -- must not be blank, must be in
-            // the range 1 to 65535.
-            
-            if ((portNumberText == null) || (portNumberText.length() < 1)) {
-                errors.add("portNumberText",
-                new ActionError("error.portNumber.required"));
-            } else {
-                try {
-                    int port = Integer.parseInt(portNumberText);
-                    if ((port <= 0) || (port >65535 ))
-                        errors.add("portNumberText", 
-                            new ActionError("error.portNumber.range"));
-                } catch (NumberFormatException e) {
+        // check for portNumber -- must not be blank, must be in
+        // the range 1 to 65535.
+        
+        if ((portNumberText == null) || (portNumberText.length() < 1)) {
+            errors.add("portNumberText",
+            new ActionMessage("error.portNumber.required"));
+        } else {
+            try {
+                int port = Integer.parseInt(portNumberText);
+                if ((port <= 0) || (port >65535 ))
                     errors.add("portNumberText", 
-                        new ActionError("error.portNumber.format"));
-                }
+                        new ActionMessage("error.portNumber.range"));
+            } catch (NumberFormatException e) {
+                errors.add("portNumberText", 
+                    new ActionMessage("error.portNumber.format"));
             }
+        }
+    
+        // shutdown text can be any non-empty string of atleast 6 characters.
         
-            // shutdown text can be any non-empty string of atleast 6 characters.
+        if ((shutdownText == null) || (shutdownText.length() < 7))
+            errors.add("shutdownText",
+            new ActionMessage("error.shutdownText.length"));
             
-            if ((shutdownText == null) || (shutdownText.length() < 7))
-                errors.add("shutdownText",
-                new ActionError("error.shutdownText.length"));
-            
-        //}
-        
         return errors;
         
     }
