@@ -1,9 +1,10 @@
 /*
- * Copyright 2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -20,29 +21,18 @@ package org.apache.webapp.admin.users;
 
 import java.io.IOException;
 import java.net.URLDecoder;
-import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.Locale;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-import javax.servlet.http.HttpSession;
+
 import org.apache.struts.action.Action;
-import org.apache.struts.action.ActionErrors;
 import org.apache.struts.action.ActionForm;
 import org.apache.struts.action.ActionForward;
 import org.apache.struts.action.ActionMapping;
 import javax.management.Attribute;
 import javax.management.MBeanServer;
-import javax.management.MBeanServerFactory;
-import javax.management.QueryExp;
-import javax.management.Query;
-import javax.management.ObjectInstance;
 import javax.management.ObjectName;
-import javax.management.JMException;
-import javax.management.MBeanAttributeInfo;
-import javax.management.MBeanOperationInfo;
-import javax.management.MBeanInfo;
 import org.apache.struts.util.MessageResources;
 import org.apache.webapp.admin.ApplicationServlet;
 import org.apache.webapp.admin.TomcatTreeBuilder;
@@ -52,7 +42,7 @@ import org.apache.webapp.admin.TomcatTreeBuilder;
  * updated User back to the underlying database.</p>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 303390 $ $Date: 2004-10-18 01:37:56 -0500 (Mon, 18 Oct 2004) $
+ * @version $Revision: 476694 $ $Date: 2006-11-18 20:41:49 -0700 (Sat, 18 Nov 2006) $
  * @since 4.1
  */
 
@@ -97,7 +87,6 @@ public final class SaveUserAction extends Action {
             mserver = ((ApplicationServlet) getServlet()).getServer();
         }
         MessageResources resources = getResources(request);
-        HttpSession session = request.getSession();
         Locale locale = getLocale(request);
 
         // Has this transaction been cancelled?
@@ -207,8 +196,8 @@ public final class SaveUserAction extends Action {
             addsig[0] = "java.lang.String";
             Object addpar[] = new Object[1];
             for (int i = 0; i < groups.length; i++) {
-                addpar[0] =
-                    (new ObjectName(groups[i])).getKeyProperty("groupname");
+                addpar[0] = ObjectName.unquote(
+                    (new ObjectName(groups[i])).getKeyProperty("groupname"));
                 mserver.invoke(oname, "addGroup",
                                addpar, addsig);
             }
