@@ -1,9 +1,10 @@
 /*
- * Copyright 2001-2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -16,26 +17,15 @@
 
 package org.apache.webapp.admin.valve;
 
-import java.util.Locale;
 import java.io.IOException;
-import javax.management.Attribute;
-import javax.management.MBeanServer;
-import javax.management.MBeanServerFactory;
-import javax.management.QueryExp;
-import javax.management.Query;
-import javax.management.ObjectName;
-import javax.management.JMException;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 import org.apache.struts.action.Action;
-import org.apache.struts.action.ActionError;
-import org.apache.struts.action.ActionErrors;
 import org.apache.struts.action.ActionForm;
 import org.apache.struts.action.ActionForward;
 import org.apache.struts.action.ActionMapping;
-import org.apache.struts.util.MessageResources;
 import org.apache.webapp.admin.ApplicationServlet;
 
 /**
@@ -43,19 +33,11 @@ import org.apache.webapp.admin.ApplicationServlet;
  * <em>Edit Valve</em> transactions for Request Dumper valve.
  *
  * @author Manveen Kaur
- * @version $Revision: 303390 $ $Date: 2004-10-18 01:37:56 -0500 (Mon, 18 Oct 2004) $
+ * @version $Revision: 479010 $ $Date: 2006-11-24 14:46:09 -0700 (Fri, 24 Nov 2006) $
  */
 
 public final class SaveRequestDumperValveAction extends Action {
 
-
-    // ----------------------------------------------------- Instance Variables
-
-    /**
-     * The MBeanServer we will be interacting with.
-     */
-    private MBeanServer mBServer = null;
-    
     // --------------------------------------------------------- Public Methods
     
     
@@ -82,36 +64,21 @@ public final class SaveRequestDumperValveAction extends Action {
         
         // Acquire the resources that we need
         HttpSession session = request.getSession();
-        Locale locale = getLocale(request);
-        MessageResources resources = getResources(request);
-        
-        // Acquire a reference to the MBeanServer containing our MBeans
-        try {
-            mBServer = ((ApplicationServlet) getServlet()).getServer();
-        } catch (Throwable t) {
-            throw new ServletException
-            ("Cannot acquire MBeanServer reference", t);
-        }
         
         // Identify the requested action
         RequestDumperValveForm vform = (RequestDumperValveForm) form;
         String adminAction = vform.getAdminAction();
-        String vObjectName = vform.getObjectName();
         String parent = vform.getParentObjectName();
         String valveType = vform.getValveType();
             
         // Perform a "Create Valve" transaction (if requested)
         if ("Create".equals(adminAction)) {
         
-            vObjectName = ValveUtil.createValve(parent, valveType, 
-                                response, request, mapping, 
+            ValveUtil.createValve(parent, valveType, response, request, mapping,
                                 (ApplicationServlet) getServlet());
            
         }
 
-        // Perform attribute updates as requested
-        String attribute = null;
-                
         // Forward to the success reporting page
         session.removeAttribute(mapping.getAttribute());
         return (mapping.findForward("Save Successful"));
